@@ -4,7 +4,6 @@ set -e
 
 export NUMBA_DEVELOPER_MODE=1
 export NUMBA_DISABLE_ERROR_MESSAGE_HIGHLIGHTING=1
-export NUMBA_CAPTURED_ERRORS="new_style"
 export PYTHONFAULTHANDLER=1
 
 # Disable NumPy dispatching to AVX512_SKX feature extensions if the chip is
@@ -38,8 +37,7 @@ if [[ "$archstr" == 'ppc64le' ]]; then
     TEST_NPROCS=16
 fi
 
-# Check Numba executables are there
-pycc -h
+# Check Numba executable is there
 numba -h
 
 # run system info tool
