@@ -9,7 +9,13 @@ from numba.core.typing.templates import (AttributeTemplate, AbstractTemplate,
 # import time side effect: array operations requires typing support of sequence
 # defined in collections: e.g. array.shape[i]
 from numba.core.typing import collections
-from numba.core.errors import TypingError
+from numba.core.errors import (TypingError, RequireLiteralValue, NumbaTypeError,
+                               NumbaNotImplementedError, NumbaAssertionError,
+                               NumbaKeyError, NumbaIndexError, NumbaValueError)
+from numba.core.cgutils import is_nonelike
+
+numpy_version = tuple(map(int, np.__version__.split('.')[:2]))
+
 
 Indexing = namedtuple("Indexing", ("index", "result", "advanced"))
 
@@ -30,45 +36,88 @@ def get_array_index_type(ary, idx):
     right_indices = []
     ellipsis_met = False
     advanced = False
-    has_integer = False
+    num_newaxis = 0
 
     if not isinstance(idx, types.BaseTuple):
         idx = [idx]
 
+    # Here, a subspace is considered as a contiguous group of advanced indices.
+    # num_subspaces keeps track of the number of such
+    # contiguous groups.
+    in_subspace = False
+    num_subspaces = 0
+    array_indices = 0
+
     # Walk indices
     for ty in idx:
         if ty is types.ellipsis:
             if ellipsis_met:
-                raise TypeError("only one ellipsis allowed in array index "
-                                "(got %s)" % (idx,))
+                raise NumbaTypeError(
+                    "Only one ellipsis allowed in array indices "
+                    "(got %s)" % (idx,))
             ellipsis_met = True
+            in_subspace = False
         elif isinstance(ty, types.SliceType):
-            pass
+            # If we encounter a non-advanced index while in a
+            # subspace then that subspace ends.
+            in_subspace = False
+        # In advanced indexing, any index broadcastable to an
+        # array is considered an advanced index. Hence all the
+        # branches below are considered as advanced indices.
         elif isinstance(ty, types.Integer):
             # Normalize integer index
             ty = types.intp if ty.signed else types.uintp
             # Integer indexing removes the given dimension
             ndim -= 1
-            has_integer = True
+            # If we're within a subspace/contiguous group of
+            # advanced indices then no action is necessary
+            # since we've already counted that subspace once.
+            if not in_subspace:
+                # If we're not within a subspace and we encounter
+                # this branch then we have a new subspace/group.
+                num_subspaces += 1
+                in_subspace = True
         elif (isinstance(ty, types.Array) and ty.ndim == 0
               and isinstance(ty.dtype, types.Integer)):
             # 0-d array used as integer index
             ndim -= 1
-            has_integer = True
+            if not in_subspace:
+                num_subspaces += 1
+                in_subspace = True
         elif (isinstance(ty, types.Array)
-              and ty.ndim == 1
               and isinstance(ty.dtype, (types.Integer, types.Boolean))):
-            if advanced or has_integer:
-                # We don't support the complicated combination of
-                # advanced indices (and integers are considered part
-                # of them by Numpy).
-                raise NotImplementedError("only one advanced index supported")
+            if ty.ndim > 1:
+                # Advanced indexing limitation # 1
+                raise NumbaTypeError(
+                    "Multi-dimensional indices are not supported.")
+            array_indices += 1
+            # The condition for activating advanced indexing is simply
+            # having at least one array with size > 1.
             advanced = True
+            if not in_subspace:
+                num_subspaces += 1
+                in_subspace = True
+        elif (is_nonelike(ty)):
+            ndim += 1
+            num_newaxis += 1
         else:
-            raise TypeError("unsupported array index type %s in %s"
-                            % (ty, idx))
+            raise NumbaTypeError("Unsupported array index type %s in %s"
+                                 % (ty, idx))
         (right_indices if ellipsis_met else left_indices).append(ty)
 
+    if advanced:
+        if array_indices > 1:
+            # Advanced indexing limitation # 2
+            msg = "Using more than one non-scalar array index is unsupported."
+            raise NumbaTypeError(msg)
+
+        if num_subspaces > 1:
+            # Advanced indexing limitation # 3
+            msg = ("Using more than one indexing subspace is unsupported."
+                   " An indexing subspace is a group of one or more"
+                   " consecutive indices comprising integer or array types.")
+            raise NumbaTypeError(msg)
+
     # Only Numpy arrays support advanced indexing
     if advanced and not isinstance(ary, types.Array):
         return
@@ -79,10 +128,10 @@ def get_array_index_type(ary, idx):
         assert right_indices[0] is types.ellipsis
         del right_indices[0]
 
-    n_indices = len(all_indices) - ellipsis_met
+    n_indices = len(all_indices) - ellipsis_met - num_newaxis
     if n_indices > ary.ndim:
-        raise TypeError("cannot index %s with %d indices: %s"
-                        % (ary, n_indices, idx))
+        raise NumbaTypeError("cannot index %s with %d indices: %s"
+                             % (ary, n_indices, idx))
     if n_indices == ary.ndim and ndim == 0 and not ellipsis_met:
         # Full integer indexing => scalar result
         # (note if ellipsis is present, a 0-d view is returned instead)
@@ -166,6 +215,7 @@ class GetItemBuffer(AbstractTemplate):
         if out is not None:
             return signature(out.result, ary, out.index)
 
+
 @infer_global(operator.setitem)
 class SetItemBuffer(AbstractTemplate):
     def generic(self, args, kws):
@@ -174,13 +224,14 @@ class SetItemBuffer(AbstractTemplate):
         if not isinstance(ary, types.Buffer):
             return
         if not ary.mutable:
-            raise TypeError("Cannot modify value of type %s" %(ary,))
+            msg = f"Cannot modify readonly array of type: {ary}"
+            raise NumbaTypeError(msg)
         out = get_array_index_type(ary, idx)
         if out is None:
             return
 
         idx = out.index
-        res = out.result
+        res = out.result  # res is the result type of the access ary[idx]
         if isinstance(res, types.Array):
             # Indexing produces an array
             if isinstance(val, types.Array):
@@ -192,7 +243,7 @@ class SetItemBuffer(AbstractTemplate):
             elif isinstance(val, types.Sequence):
                 if (res.ndim == 1 and
                     self.context.can_convert(val.dtype, res.dtype)):
-                    # Allow assignement of sequence to 1d array
+                    # Allow assignment of sequence to 1d array
                     res = val
                 else:
                     # NOTE: sequence-to-array broadcasting is unsupported
@@ -215,6 +266,10 @@ class SetItemBuffer(AbstractTemplate):
                 else:
                     return
             res = val
+        elif (isinstance(val, types.Array) and val.ndim == 0
+              and self.context.can_convert(val.dtype, res)):
+            # val is an array(T, 0d, O), where T is the type of res, O is order
+            res = val
         else:
             return
         return signature(types.none, ary, idx, res)
@@ -238,6 +293,9 @@ class ArrayAttribute(AttributeTemplate):
     def resolve_dtype(self, ary):
         return types.DType(ary.dtype)
 
+    def resolve_nbytes(self, ary):
+        return types.intp
+
     def resolve_itemsize(self, ary):
         return types.intp
 
@@ -294,7 +352,8 @@ class ArrayAttribute(AttributeTemplate):
             if ty in types.number_domain:
                 # Guard against non integer type
                 if not isinstance(ty, types.Integer):
-                    raise TypeError("transpose() arg cannot be {0}".format(ty))
+                    msg = "transpose() arg cannot be {0}".format(ty)
+                    raise TypingError(msg)
                 return True
             else:
                 return False
@@ -322,8 +381,9 @@ class ArrayAttribute(AttributeTemplate):
 
         else:
             if any(not sentry_shape_scalar(a) for a in args):
-                raise TypeError("transpose({0}) is not supported".format(
-                    ', '.join(args)))
+                msg = "transpose({0}) is not supported".format(
+                    ', '.join(args))
+                raise TypingError(msg)
             assert ary.ndim == len(args)
             return signature(self.resolve_T(ary).copy(layout="A"), *args)
 
@@ -343,19 +403,25 @@ class ArrayAttribute(AttributeTemplate):
         if not args:
             return signature(ary.dtype)
 
-    @bound_function("array.itemset")
-    def resolve_itemset(self, ary, args, kws):
-        assert not kws
-        # We don't support explicit arguments as that's exactly equivalent
-        # to regular indexing.  The no-argument form is interesting to
-        # allow some degree of genericity when writing functions.
-        if len(args) == 1:
-            return signature(types.none, ary.dtype)
+    if numpy_version < (2, 0):
+        @bound_function("array.itemset")
+        def resolve_itemset(self, ary, args, kws):
+            assert not kws
+            # We don't support explicit arguments as that's exactly equivalent
+            # to regular indexing.  The no-argument form is interesting to
+            # allow some degree of genericity when writing functions.
+            if len(args) == 1:
+                return signature(types.none, ary.dtype)
 
     @bound_function("array.nonzero")
     def resolve_nonzero(self, ary, args, kws):
         assert not args
         assert not kws
+        if ary.ndim == 0 and numpy_version >= (2, 1):
+            raise NumbaValueError(
+                "Calling nonzero on 0d arrays is not allowed."
+                " Use np.atleast_1d(scalar).nonzero() instead."
+            )
         # 0-dim arrays return one result array
         ndim = max(ary.ndim, 1)
         retty = types.UniTuple(types.Array(types.intp, 1, 'C'), ndim)
@@ -367,7 +433,7 @@ class ArrayAttribute(AttributeTemplate):
             if ty in types.number_domain:
                 # Guard against non integer type
                 if not isinstance(ty, types.Integer):
-                    raise TypeError("reshape() arg cannot be {0}".format(ty))
+                    raise TypingError("reshape() arg cannot be {0}".format(ty))
                 return True
             else:
                 return False
@@ -375,7 +441,7 @@ class ArrayAttribute(AttributeTemplate):
         assert not kws
         if ary.layout not in 'CF':
             # only work for contiguous array
-            raise TypeError("reshape() supports contiguous array only")
+            raise TypingError("reshape() supports contiguous array only")
 
         if len(args) == 1:
             # single arg
@@ -393,12 +459,12 @@ class ArrayAttribute(AttributeTemplate):
 
         elif len(args) == 0:
             # no arg
-            raise TypeError("reshape() take at least one arg")
+            raise TypingError("reshape() take at least one arg")
 
         else:
             # vararg case
             if any(not sentry_shape_scalar(a) for a in args):
-                raise TypeError("reshape({0}) is not supported".format(
+                raise TypingError("reshape({0}) is not supported".format(
                     ', '.join(map(str, args))))
 
             retty = ary.copy(ndim=len(args))
@@ -408,8 +474,7 @@ class ArrayAttribute(AttributeTemplate):
     def resolve_sort(self, ary, args, kws):
         assert not args
         assert not kws
-        if ary.ndim == 1:
-            return signature(types.none)
+        return signature(types.none)
 
     @bound_function("array.argsort")
     def resolve_argsort(self, ary, args, kws):
@@ -417,7 +482,7 @@ class ArrayAttribute(AttributeTemplate):
         kwargs = dict(kws)
         kind = kwargs.pop('kind', types.StringLiteral('quicksort'))
         if not isinstance(kind, types.StringLiteral):
-            raise errors.TypingError('"kind" must be a string literal')
+            raise TypingError('"kind" must be a string literal')
         if kwargs:
             msg = "Unsupported keywords: {!r}"
             raise TypingError(msg.format([k for k in kwargs.keys()]))
@@ -444,13 +509,16 @@ class ArrayAttribute(AttributeTemplate):
         from .npydecl import parse_dtype
         assert not kws
         dtype, = args
+        if isinstance(dtype, types.UnicodeType):
+            raise RequireLiteralValue(("array.astype if dtype is a string it "
+                                       "must be constant"))
         dtype = parse_dtype(dtype)
         if dtype is None:
             return
         if not self.context.can_convert(ary.dtype, dtype):
-            raise TypeError("astype(%s) not supported on %s: "
-                            "cannot convert from %s to %s"
-                            % (dtype, ary, ary.dtype, dtype))
+            raise TypingError("astype(%s) not supported on %s: "
+                              "cannot convert from %s to %s"
+                              % (dtype, ary, ary.dtype, dtype))
         layout = ary.layout if ary.layout in 'CF' else 'C'
         # reset the write bit irrespective of whether the cast type is the same
         # as the current dtype, this replicates numpy
@@ -462,36 +530,36 @@ class ArrayAttribute(AttributeTemplate):
         # Only support no argument version (default order='C')
         assert not kws
         assert not args
-        return signature(ary.copy(ndim=1, layout='C'))
+        copy_will_be_made = ary.layout != 'C'
+        readonly = not (copy_will_be_made or ary.mutable)
+        return signature(ary.copy(ndim=1, layout='C', readonly=readonly))
 
     @bound_function("array.flatten")
     def resolve_flatten(self, ary, args, kws):
         # Only support no argument version (default order='C')
         assert not kws
         assert not args
-        return signature(ary.copy(ndim=1, layout='C'))
-
-    @bound_function("array.take")
-    def resolve_take(self, ary, args, kws):
-        assert not kws
-        argty, = args
-        if isinstance(argty, types.Integer):
-            sig = signature(ary.dtype, *args)
-        elif isinstance(argty, types.Array):
-            sig = signature(argty.copy(layout='C', dtype=ary.dtype), *args)
-        elif isinstance(argty, types.List): # 1d lists only
-            sig = signature(types.Array(ary.dtype, 1, 'C'), *args)
-        elif isinstance(argty, types.BaseTuple):
-            sig = signature(types.Array(ary.dtype, np.ndim(argty), 'C'), *args)
-        else:
-            raise TypeError("take(%s) not supported for %s" % argty)
-        return sig
+        # To ensure that Numba behaves exactly like NumPy,
+        # we also clear the read-only flag when doing a "flatten"
+        # Why? Two reasons:
+        # Because flatten always returns a copy. (see NumPy docs for "flatten")
+        # And because a copy always returns a writeable array.
+        # ref: https://numpy.org/doc/stable/reference/generated/numpy.copy.html
+        return signature(ary.copy(ndim=1, layout='C', readonly=False))
 
     def generic_resolve(self, ary, attr):
         # Resolution of other attributes, for record arrays
         if isinstance(ary.dtype, types.Record):
             if attr in ary.dtype.fields:
-                return ary.copy(dtype=ary.dtype.typeof(attr), layout='A')
+                attr_dtype = ary.dtype.typeof(attr)
+                if isinstance(attr_dtype, types.NestedArray):
+                    return ary.copy(
+                        dtype=attr_dtype.dtype,
+                        ndim=ary.ndim + attr_dtype.ndim,
+                        layout='A'
+                    )
+                else:
+                    return ary.copy(dtype=attr_dtype, layout='A')
 
 
 @infer_getattr
@@ -511,6 +579,7 @@ class DTypeAttr(AttributeTemplate):
             return None  # other types not supported yet
         return types.StringLiteral(val)
 
+
 @infer
 class StaticGetItemArray(AbstractTemplate):
     key = "static_getitem"
@@ -519,10 +588,19 @@ class StaticGetItemArray(AbstractTemplate):
         # Resolution of members for record and structured arrays
         ary, idx = args
         if (isinstance(ary, types.Array) and isinstance(idx, str) and
-            isinstance(ary.dtype, types.Record)):
+                isinstance(ary.dtype, types.Record)):
             if idx in ary.dtype.fields:
-                ret = ary.copy(dtype=ary.dtype.typeof(idx), layout='A')
-                return signature(ret, *args)
+                attr_dtype = ary.dtype.typeof(idx)
+                if isinstance(attr_dtype, types.NestedArray):
+                    ret = ary.copy(
+                        dtype=attr_dtype.dtype,
+                        ndim=ary.ndim + attr_dtype.ndim,
+                        layout='A'
+                    )
+                    return signature(ret, *args)
+                else:
+                    ret = ary.copy(dtype=attr_dtype, layout='A')
+                    return signature(ret, *args)
 
 
 @infer_getattr
@@ -534,6 +612,7 @@ class RecordAttribute(AttributeTemplate):
         assert ret
         return ret
 
+
 @infer
 class StaticGetItemRecord(AbstractTemplate):
     key = "static_getitem"
@@ -543,8 +622,9 @@ class StaticGetItemRecord(AbstractTemplate):
         record, idx = args
         if isinstance(record, types.Record) and isinstance(idx, str):
             if idx not in record.fields:
-                raise KeyError(f"Field '{idx}' was not found in record with "
-                               f"fields {tuple(record.fields.keys())}")
+                raise NumbaKeyError(f"Field '{idx}' was not found in record "
+                                    "with fields "
+                                    f"{tuple(record.fields.keys())}")
             ret = record.typeof(idx)
             assert ret
             return signature(ret, *args)
@@ -555,13 +635,23 @@ class StaticGetItemLiteralRecord(AbstractTemplate):
     def generic(self, args, kws):
         # Resolution of members for records
         record, idx = args
-        if isinstance(record, types.Record) and isinstance(idx, types.StringLiteral):
-            if idx.literal_value not in record.fields:
-                raise KeyError(f"Field '{idx.literal_value}' was not found in record with "
-                               f"fields {tuple(record.fields.keys())}")
-            ret = record.typeof(idx.literal_value)
-            assert ret
-            return signature(ret, *args)
+        if isinstance(record, types.Record):
+            if isinstance(idx, types.StringLiteral):
+                if idx.literal_value not in record.fields:
+                    msg = (f"Field '{idx.literal_value}' was not found in "
+                           f"record with fields {tuple(record.fields.keys())}")
+                    raise NumbaKeyError(msg)
+                ret = record.typeof(idx.literal_value)
+                assert ret
+                return signature(ret, *args)
+            elif isinstance(idx, types.IntegerLiteral):
+                if idx.literal_value >= len(record.fields):
+                    msg = f"Requested index {idx.literal_value} is out of range"
+                    raise NumbaIndexError(msg)
+                field_names = list(record.fields)
+                ret = record.typeof(field_names[idx.literal_value])
+                assert ret
+                return signature(ret, *args)
 
 
 @infer
@@ -571,10 +661,21 @@ class StaticSetItemRecord(AbstractTemplate):
     def generic(self, args, kws):
         # Resolution of members for record and structured arrays
         record, idx, value = args
-        if isinstance(record, types.Record) and isinstance(idx, str):
-            expectedty = record.typeof(idx)
-            if self.context.can_convert(value, expectedty) is not None:
-                return signature(types.void, record, types.literal(idx), value)
+        if isinstance(record, types.Record):
+            if isinstance(idx, str):
+                expectedty = record.typeof(idx)
+                if self.context.can_convert(value, expectedty) is not None:
+                    return signature(types.void, record, types.literal(idx),
+                                     value)
+            elif isinstance(idx, int):
+                if idx >= len(record.fields):
+                    msg = f"Requested index {idx} is out of range"
+                    raise NumbaIndexError(msg)
+                str_field = list(record.fields)[idx]
+                expectedty = record.typeof(str_field)
+                if self.context.can_convert(value, expectedty) is not None:
+                    return signature(types.void, record, types.literal(idx),
+                                     value)
 
 
 @infer_global(operator.setitem)
@@ -584,8 +685,9 @@ class StaticSetItemLiteralRecord(AbstractTemplate):
         target, idx, value = args
         if isinstance(target, types.Record) and isinstance(idx, types.StringLiteral):
             if idx.literal_value not in target.fields:
-                raise KeyError(f"Field '{idx.literal_value}' was not found in record with "
-                               f"fields {tuple(target.fields.keys())}")
+                msg = (f"Field '{idx.literal_value}' was not found in record "
+                       f"with fields {tuple(target.fields.keys())}")
+                raise NumbaKeyError(msg)
             expectedty = target.typeof(idx.literal_value)
             if self.context.can_convert(value, expectedty) is not None:
                 return signature(types.void, target, idx, value)
@@ -634,8 +736,11 @@ def _expand_integer(ty):
 
 
 def generic_homog(self, args, kws):
-    assert not args
-    assert not kws
+    if args:
+        raise NumbaAssertionError("args not supported")
+    if kws:
+        raise NumbaAssertionError("kws not supported")
+
     return signature(self.this.dtype, recvr=self.this)
 
 
@@ -681,7 +786,7 @@ def sum_expand(self, args, kws):
         # There is an axis parameter, either arg or kwarg
         if self.this.ndim == 1:
             # 1d reduces to a scalar
-            return_type = self.this.dtype
+            return_type = _expand_integer(self.this.dtype)
         else:
             # the return type of this summation is  an array of dimension one
             # less than the input array.
@@ -715,8 +820,10 @@ def sum_expand(self, args, kws):
 
 
 def generic_expand_cumulative(self, args, kws):
-    assert not args
-    assert not kws
+    if args:
+        raise NumbaAssertionError("args unsupported")
+    if kws:
+        raise NumbaAssertionError("kwargs unsupported")
     assert isinstance(self.this, types.Array)
     return_type = types.Array(dtype=_expand_integer(self.this.dtype),
                               ndim=1, layout='C')
@@ -730,6 +837,7 @@ def generic_hetero_real(self, args, kws):
         return signature(types.float64, recvr=self.this)
     return signature(self.this.dtype, recvr=self.this)
 
+
 def generic_hetero_always_real(self, args, kws):
     assert not args
     assert not kws
@@ -739,46 +847,27 @@ def generic_hetero_always_real(self, args, kws):
         return signature(self.this.dtype.underlying_float, recvr=self.this)
     return signature(self.this.dtype, recvr=self.this)
 
+
 def generic_index(self, args, kws):
     assert not args
     assert not kws
     return signature(types.intp, recvr=self.this)
 
+
 def install_array_method(name, generic, prefer_literal=True):
     my_attr = {"key": "array." + name, "generic": generic,
                "prefer_literal": prefer_literal}
     temp_class = type("Array_" + name, (AbstractTemplate,), my_attr)
+
     def array_attribute_attachment(self, ary):
         return types.BoundFunction(temp_class, ary)
 
     setattr(ArrayAttribute, "resolve_" + name, array_attribute_attachment)
 
-# Functions that return the same type as the array
-for fname in ["min", "max"]:
-    install_array_method(fname, generic_homog)
 
 # Functions that return a machine-width type, to avoid overflows
-install_array_method("prod", generic_expand)
 install_array_method("sum", sum_expand, prefer_literal=True)
 
-# Functions that return a machine-width type, to avoid overflows
-for fname in ["cumsum", "cumprod"]:
-    install_array_method(fname, generic_expand_cumulative)
-
-# Functions that require integer arrays get promoted to float64 return
-for fName in ["mean"]:
-    install_array_method(fName, generic_hetero_real)
-
-# var and std by definition return in real space and int arrays
-# get promoted to float64 return
-for fName in ["var", "std"]:
-    install_array_method(fName, generic_hetero_always_real)
-
-
-# Functions that return an index (intp)
-install_array_method("argmin", generic_index)
-install_array_method("argmax", generic_index)
-
 
 @infer_global(operator.eq)
 class CmpOpEqArray(AbstractTemplate):
