@@ -4,21 +4,21 @@ from copy import deepcopy, copy
 import warnings
 
 from numba.core.compiler_machinery import (FunctionPass, AnalysisPass,
-                                           register_pass)
+                                           SSACompliantMixin, register_pass)
 from numba.core import (errors, types, ir, bytecode, postproc, rewrites, config,
-                        transforms)
+                        transforms, consts)
 from numba.misc.special import literal_unroll
 from numba.core.analysis import (dead_branch_prune, rewrite_semantic_constants,
                                  find_literally_calls, compute_cfg_from_blocks,
                                  compute_use_defs)
-from numba.core.inline_closurecall import (InlineClosureCallPass,
-                                           inline_closure_call)
 from numba.core.ir_utils import (guard, resolve_func_from_module, simplify_CFG,
                                  GuardException, convert_code_obj_to_function,
-                                 mk_unique_var, build_definitions,
+                                 build_definitions,
                                  replace_var_names, get_name_var_table,
                                  compile_to_numba_ir, get_definition,
-                                 find_max_label, rename_labels)
+                                 find_max_label, rename_labels,
+                                 transfer_scope, fixup_var_define_in_scope,
+                                 )
 from numba.core.ssa import reconstruct_ssa
 from numba.core import interpreter
 
@@ -153,7 +153,7 @@ class RewriteSemanticConstants(FunctionPass):
 
 
 @register_pass(mutates_CFG=True, analysis_only=False)
-class DeadBranchPrune(FunctionPass):
+class DeadBranchPrune(SSACompliantMixin, FunctionPass):
     _name = "dead_branch_prune"
 
     def __init__(self):
@@ -197,6 +197,7 @@ class InlineClosureLikes(FunctionPass):
         # no ability to resolve certain typed function calls in the array
         # inlining code, use this variable to indicate
         typed_pass = not isinstance(state.return_type, types.misc.PyObject)
+        from numba.core.inline_closurecall import InlineClosureCallPass
         inline_pass = InlineClosureCallPass(
             state.func_ir,
             state.flags.auto_parallel,
@@ -208,10 +209,8 @@ class InlineClosureLikes(FunctionPass):
         post_proc = postproc.PostProcessor(state.func_ir)
         post_proc.run()
 
-        if config.DEBUG or config.DUMP_IR:
-            name = state.func_ir.func_id.func_qualname
-            print(("IR DUMP: %s" % name).center(80, "-"))
-            state.func_ir.dump()
+        fixup_var_define_in_scope(state.func_ir.blocks)
+
         return True
 
 
@@ -592,7 +591,6 @@ class MakeFunctionToJitFunction(FunctionPass):
                                 ok = all([isinstance(getdef(x), ir.Const)
                                           for x in kw_default.items])
                             if not ok:
-                                print("NOT OK")
                                 continue
 
                             pyfunc = convert_code_obj_to_function(node, func_ir)
@@ -868,7 +866,9 @@ class MixedContainerUnroller(FunctionPass):
                         if (isinstance(stmt.value, ir.Expr) and
                                 stmt.value.op == "typed_getitem"):
                             if isinstance(branch_ty, types.Literal):
-                                new_const_name = mk_unique_var("branch_const")
+                                scope = switch_ir.blocks[lbl].scope
+                                new_const_name = scope.redefine(
+                                    "branch_const", stmt.loc).name
                                 new_const_var = ir.Var(
                                     blk.scope, new_const_name, stmt.loc)
                                 new_const_val = ir.Const(
@@ -905,7 +905,17 @@ class MixedContainerUnroller(FunctionPass):
 
             new_var_dict = {}
             for name, var in var_table.items():
-                new_var_dict[name] = mk_unique_var(name)
+                scope = switch_ir.blocks[lbl].scope
+                try:
+                    scope.get_exact(name)
+                except errors.NotDefinedError:
+                    # In case the scope doesn't have the variable, we need to
+                    # define it prior creating new copies of it! This is
+                    # because the scope of the function and the scope of the
+                    # loop are different and the variable needs to be redefined
+                    # within the scope of the loop.
+                    scope.define(name, var.loc)
+                new_var_dict[name] = scope.redefine(name, var.loc).name
             replace_var_names(loop_blocks, new_var_dict)
 
             # clobber the sentinel body and then stuff in the rest
@@ -944,9 +954,51 @@ class MixedContainerUnroller(FunctionPass):
         """
         elif_tplt = "\n\telif PLACEHOLDER_INDEX in (%s,):\n\t\tSENTINEL = None"
 
+        # Note regarding the insertion of the garbage/defeat variables below:
+        # These values have been designed and inserted to defeat a specific
+        # behaviour of the cpython optimizer. The optimization was introduced
+        # in Python 3.10.
+
+        # The URL for the BPO is:
+        # https://bugs.python.org/issue44626
+        # The code for the optimization can be found at:
+        # https://github.com/python/cpython/blob/d41abe8/Python/compile.c#L7533-L7557
+
+        # Essentially the CPython optimizer will inline the exit block under
+        # certain circumstances and thus replace the jump with a return if the
+        # exit block is small enough.  This is an issue for unroller, as it
+        # looks for a jump, not a return, when it inserts the generated switch
+        # table.
+
+        # Part of the condition for this optimization to be applied is that the
+        # exit block not exceed a certain (4 at the time of writing) number of
+        # bytecode instructions. We defeat the optimizer by inserting a
+        # sufficient number of instructions so that the exit block is big
+        # enough. We don't care about this garbage, because the generated exit
+        # block is discarded anyway when we smash the switch table into the
+        # original function and so all the inserted garbage is dropped again.
+
+        # The final lines of the stacktrace w/o this will look like:
+        #
+        #  File "/numba/numba/core/untyped_passes.py", line 830, \
+        #       in inject_loop_body
+        #   sentinel_exits.add(blk.body[-1].target)
+        # AttributeError: Failed in nopython mode pipeline \
+        #       (step: handles literal_unroll)
+        # Failed in literal_unroll_subpipeline mode pipeline \
+        #       (step: performs mixed container unroll)
+        # 'Return' object has no attribute 'target'
+        #
+        # Which indicates that a Return has been found instead of a Jump
+
         b = ('def foo():\n\tif PLACEHOLDER_INDEX in (%s,):\n\t\t'
              'SENTINEL = None\n%s\n\telse:\n\t\t'
-             'raise RuntimeError("Unreachable")')
+             'raise RuntimeError("Unreachable")\n\t'
+             'py310_defeat1 = 1\n\t'
+             'py310_defeat2 = 2\n\t'
+             'py310_defeat3 = 3\n\t'
+             'py310_defeat4 = 4\n\t'
+             )
         keys = [k for k in data.keys()]
 
         elifs = []
@@ -975,7 +1027,6 @@ class MixedContainerUnroller(FunctionPass):
 
         # 0. Find the loops containing literal_unroll and store this
         #    information
-        literal_unroll_info = dict()
         unroll_info = namedtuple(
             "unroll_info", [
                 "loop", "call", "arg", "getitem"])
@@ -996,116 +1047,145 @@ class MixedContainerUnroller(FunctionPass):
                 raise GuardException
             return some_call
 
-        for lbl, loop in loops.items():
-            # TODO: check the loop head has literal_unroll, if it does but
-            # does not conform to the following then raise
+        def find_unroll_loops(loops):
+            """This finds loops which are compliant with the form:
+            for i in range(len(literal_unroll(<something>>)))"""
+            unroll_loops = {}
+            for header_lbl, loop in loops.items():
+                # TODO: check the loop head has literal_unroll, if it does but
+                # does not conform to the following then raise
 
-            # scan loop header
-            iternexts = [_ for _ in
-                         func_ir.blocks[loop.header].find_exprs('iternext')]
-            if len(iternexts) != 1:
-                return False
-            for iternext in iternexts:
-                # Walk the canonicalised loop structure and check it
-                # Check loop form range(literal_unroll(container)))
-                phi = guard(get_definition, func_ir,  iternext.value)
-                if phi is None:
+                # scan loop header
+                iternexts = [_ for _ in
+                             func_ir.blocks[loop.header].find_exprs('iternext')]
+                # needs to be an single iternext driven loop
+                if len(iternexts) != 1:
                     continue
+                for iternext in iternexts:
+                    # Walk the canonicalised loop structure and check it
+                    # Check loop form range(literal_unroll(container)))
+                    phi = guard(get_definition, func_ir,  iternext.value)
+                    if phi is None:
+                        continue
 
-                # check call global "range"
-                range_call = guard(get_call_args, phi.value, range)
-                if range_call is None:
-                    continue
-                range_arg = range_call.args[0]
+                    # check call global "range"
+                    range_call = guard(get_call_args, phi.value, range)
+                    if range_call is None:
+                        continue
+                    range_arg = range_call.args[0]
 
-                # check call global "len"
-                len_call = guard(get_call_args, range_arg, len)
-                if len_call is None:
-                    continue
-                len_arg = len_call.args[0]
+                    # check call global "len"
+                    len_call = guard(get_call_args, range_arg, len)
+                    if len_call is None:
+                        continue
+                    len_arg = len_call.args[0]
 
-                # check literal_unroll
-                literal_unroll_call = guard(get_definition, func_ir, len_arg)
-                if literal_unroll_call is None:
-                    continue
-                if not isinstance(literal_unroll_call, ir.Expr):
-                    continue
-                if literal_unroll_call.op != "call":
-                    continue
-                literal_func = getattr(literal_unroll_call, 'func', None)
-                if not literal_func:
-                    continue
-                call_func = guard(get_definition, func_ir,
-                                  literal_unroll_call.func)
-                if call_func is None:
-                    continue
-                call_func = call_func.value
+                    # check literal_unroll
+                    literal_unroll_call = guard(get_definition, func_ir,
+                                                len_arg)
+                    if literal_unroll_call is None:
+                        continue
+                    if not isinstance(literal_unroll_call, ir.Expr):
+                        continue
+                    if literal_unroll_call.op != "call":
+                        continue
+                    literal_func = getattr(literal_unroll_call, 'func', None)
+                    if not literal_func:
+                        continue
+                    call_func = guard(get_definition, func_ir,
+                                      literal_unroll_call.func)
+                    if call_func is None:
+                        continue
+                    call_func_value = call_func.value
 
-                if call_func is literal_unroll:
-                    assert len(literal_unroll_call.args) == 1
-                    arg = literal_unroll_call.args[0]
-                    typemap = state.typemap
-                    resolved_arg = guard(get_definition, func_ir, arg,
-                                         lhs_only=True)
-                    ty = typemap[resolved_arg.name]
-                    assert isinstance(ty, self._accepted_types)
-                    # loop header is spelled ok, now make sure the body
-                    # actually contains a getitem
+                    if call_func_value is literal_unroll:
+                        assert len(literal_unroll_call.args) == 1
+                        unroll_loops[loop] = literal_unroll_call
+            return unroll_loops
 
-                    # find a "getitem"
-                    tuple_getitem = None
-                    for lbl in loop.body:
-                        blk = func_ir.blocks[lbl]
-                        for stmt in blk.body:
-                            if isinstance(stmt, ir.Assign):
-                                if (isinstance(stmt.value, ir.Expr) and
-                                        stmt.value.op == "getitem"):
-                                    # check for something like a[i]
-                                    if stmt.value.value != arg:
-                                        # that failed, so check for the
-                                        # definition
-                                        dfn = guard(get_definition, func_ir,
-                                                    stmt.value.value)
-                                        if dfn is None:
-                                            continue
-                                        try:
-                                            args = getattr(dfn, 'args', False)
-                                        except KeyError:
-                                            continue
-                                        if not args:
-                                            continue
-                                        if not args[0] == arg:
-                                            continue
-                                    target_ty = state.typemap[arg.name]
-                                    if not isinstance(target_ty,
-                                                      self._accepted_types):
+        def ensure_no_nested_unroll(unroll_loops):
+            # Validate loop nests, nested literal_unroll loops are unsupported.
+            # This doesn't check that there's a getitem or anything else
+            # required for the transform to work, simply just that there's no
+            # nesting.
+            for test_loop in unroll_loops:
+                for ref_loop in unroll_loops:
+                    if test_loop == ref_loop:  # comparing to self! skip
+                        continue
+                    if test_loop.header in ref_loop.body:
+                        msg = ("Nesting of literal_unroll is unsupported")
+                        loc = func_ir.blocks[test_loop.header].loc
+                        raise errors.UnsupportedError(msg, loc)
+
+        def collect_literal_unroll_info(literal_unroll_loops):
+            """Finds the loops induced by `literal_unroll`, returns a list of
+            unroll_info namedtuples for use in the transform pass.
+            """
+
+            literal_unroll_info = []
+            for loop, literal_unroll_call in literal_unroll_loops.items():
+                arg = literal_unroll_call.args[0]
+                typemap = state.typemap
+                resolved_arg = guard(get_definition, func_ir, arg,
+                                     lhs_only=True)
+                ty = typemap[resolved_arg.name]
+                assert isinstance(ty, self._accepted_types)
+                # loop header is spelled ok, now make sure the body
+                # actually contains a getitem
+
+                # find a "getitem"... only looks in the blocks that belong
+                # _solely_ to this literal_unroll (there should not be nested
+                # literal_unroll loops, this is unsupported).
+                tuple_getitem = None
+                for lbli in loop.body:
+                    blk = func_ir.blocks[lbli]
+                    for stmt in blk.body:
+                        if isinstance(stmt, ir.Assign):
+                            if (isinstance(stmt.value, ir.Expr) and
+                                    stmt.value.op == "getitem"):
+                                # check for something like a[i]
+                                if stmt.value.value != arg:
+                                    # that failed, so check for the
+                                    # definition
+                                    dfn = guard(get_definition, func_ir,
+                                                stmt.value.value)
+                                    if dfn is None:
                                         continue
-                                    tuple_getitem = stmt
-                                    break
-                        if tuple_getitem:
-                            break
-                    else:
-                        continue  # no getitem in this loop
+                                    try:
+                                        args = getattr(dfn, 'args', False)
+                                    except KeyError:
+                                        continue
+                                    if not args:
+                                        continue
+                                    if not args[0] == arg:
+                                        continue
+                                target_ty = state.typemap[arg.name]
+                                if not isinstance(target_ty,
+                                                  self._accepted_types):
+                                    continue
+                                tuple_getitem = stmt
+                                break
+                    if tuple_getitem:
+                        break
+                else:
+                    continue  # no getitem in this loop
 
-                    ui = unroll_info(loop, literal_unroll_call, arg,
-                                     tuple_getitem)
-                    literal_unroll_info[lbl] = ui
+                ui = unroll_info(loop, literal_unroll_call, arg,
+                                 tuple_getitem)
+                literal_unroll_info.append(ui)
+            return literal_unroll_info
 
+        # 1. Collect info about the literal_unroll loops, ensure they are legal
+        literal_unroll_loops = find_unroll_loops(loops)
+        # validate
+        ensure_no_nested_unroll(literal_unroll_loops)
+        # assemble info
+        literal_unroll_info = collect_literal_unroll_info(literal_unroll_loops)
         if not literal_unroll_info:
             return False
 
-        # 1. Validate loops, must not have any calls to literal_unroll
-        for test_lbl, test_loop in literal_unroll_info.items():
-            for ref_lbl, ref_loop in literal_unroll_info.items():
-                if test_lbl == ref_lbl:  # comparing to self! skip
-                    continue
-                if test_loop.loop.header in ref_loop.loop.body:
-                    msg = ("Nesting of literal_unroll is unsupported")
-                    loc = func_ir.blocks[test_loop.loop.header].loc
-                    raise errors.UnsupportedError(msg, loc)
-
         # 2. Do the unroll, get a loop and process it!
-        lbl, info = literal_unroll_info.popitem()
+        info = literal_unroll_info[0]
         self.unroll_loop(state, info)
 
         # 3. Rebuild the state, the IR has taken a hammering
@@ -1211,15 +1291,16 @@ class MixedContainerUnroller(FunctionPass):
 
         # 6. Patch in the unrolled body and fix up
         blks = state.func_ir.blocks
+        the_scope = next(iter(blks.values())).scope
         orig_lbl = tuple(this_loop_body)
 
         replace, *delete = orig_lbl
         unroll, header_block = unrolled_body, this_loop.header
         unroll_lbl = [x for x in sorted(unroll.blocks.keys())]
-        blks[replace] = unroll.blocks[unroll_lbl[0]]
+        blks[replace] = transfer_scope(unroll.blocks[unroll_lbl[0]], the_scope)
         [blks.pop(d) for d in delete]
         for k in unroll_lbl[1:]:
-            blks[k] = unroll.blocks[k]
+            blks[k] = transfer_scope(unroll.blocks[k], the_scope)
         # stitch up the loop predicate true -> new loop body jump
         blks[header_block].body[-1].truebr = replace
 
@@ -1240,7 +1321,7 @@ class MixedContainerUnroller(FunctionPass):
         # 3. No multiple mix-tuple use
 
         # keep running the transform loop until it reports no more changes
-        while(True):
+        while (True):
             stat = self.apply_transform(state)
             mutated |= stat
             if not stat:
@@ -1317,14 +1398,11 @@ class IterLoopCanonicalization(FunctionPass):
         def get_range(a):
             return range(len(a))
 
-        def tokenise(x):
-            return mk_unique_var("CANONICALISER_%s" % x)
-
         iternext = [_ for _ in
                     func_ir.blocks[loop.header].find_exprs('iternext')][0]
         LOC = func_ir.blocks[loop.header].loc
-        get_range_var = ir.Var(func_ir.blocks[loop.header].scope,
-                               tokenise('get_range_gbl'), LOC)
+        scope = func_ir.blocks[loop.header].scope
+        get_range_var = scope.redefine("CANONICALISER_get_range_gbl", LOC)
         get_range_global = ir.Global('get_range', get_range, LOC)
         assgn = ir.Assign(get_range_global, get_range_var, LOC)
 
@@ -1348,14 +1426,14 @@ class IterLoopCanonicalization(FunctionPass):
             raise ValueError("problem")
 
         # create a range(len(tup)) and inject it
-        call_get_range_var = ir.Var(entry_block.scope,
-                                    tokenise('call_get_range'), LOC)
+        call_get_range_var = scope.redefine('CANONICALISER_call_get_range', LOC)
         make_call = ir.Expr.call(get_range_var, (stmt.value.value,), (), LOC)
         assgn_call = ir.Assign(make_call, call_get_range_var, LOC)
         entry_block.body.insert(idx, assgn_call)
         entry_block.body[idx + 1].value.value = call_get_range_var
 
         glbls = copy(func_ir.func_id.func.__globals__)
+        from numba.core.inline_closurecall import inline_closure_call
         inline_closure_call(func_ir, glbls, entry_block, idx, get_range,)
         kill = entry_block.body.index(assgn)
         entry_block.body.pop(kill)
@@ -1422,6 +1500,167 @@ class IterLoopCanonicalization(FunctionPass):
         return mutated
 
 
+@register_pass(mutates_CFG=False, analysis_only=False)
+class PropagateLiterals(FunctionPass):
+    """Implement literal propagation based on partial type inference"""
+    _name = "PropagateLiterals"
+
+    def __init__(self):
+        FunctionPass.__init__(self)
+
+    def get_analysis_usage(self, AU):
+        AU.add_required(ReconstructSSA)
+
+    def run_pass(self, state):
+        func_ir = state.func_ir
+        typemap = state.typemap
+        flags = state.flags
+
+        accepted_functions = ('isinstance', 'hasattr')
+
+        if not hasattr(func_ir, '_definitions') \
+                and not flags.enable_ssa:
+            func_ir._definitions = build_definitions(func_ir.blocks)
+
+        changed = False
+
+        for block in func_ir.blocks.values():
+            for assign in block.find_insts(ir.Assign):
+                value = assign.value
+                if isinstance(value, (ir.Arg, ir.Const, ir.FreeVar, ir.Global)):
+                    continue
+
+                # 1) Don't change return stmt in the form
+                # $return_xyz = cast(value=ABC)
+                # 2) Don't propagate literal values that are not primitives
+                if isinstance(value, ir.Expr) and \
+                        value.op in ('cast', 'build_map', 'build_list',
+                                     'build_tuple', 'build_set'):
+                    continue
+
+                target = assign.target
+                if not flags.enable_ssa:
+                    # SSA is disabled when doing inlining
+                    if guard(get_definition, func_ir, target.name) is None:  # noqa: E501
+                        continue
+
+                # Numba cannot safely determine if an isinstance call
+                # with a PHI node is True/False. For instance, in
+                # the case below, the partial type inference step can coerce
+                # '$z' to float, so any call to 'isinstance(z, int)' would fail.
+                #
+                #   def fn(x):
+                #       if x > 4:
+                #           z = 1
+                #       else:
+                #           z = 3.14
+                #       if isinstance(z, int):
+                #           print('int')
+                #       else:
+                #           print('float')
+                #
+                # At the moment, one avoid propagating the literal
+                # value if the argument is a PHI node
+
+                if isinstance(value, ir.Expr) and value.op == 'call':
+
+                    fn = guard(get_definition, func_ir, value.func.name)
+                    if fn is None:
+                        continue
+
+                    if not (isinstance(fn, ir.Global) and fn.name in
+                            accepted_functions):
+                        continue
+
+                    for arg in value.args:
+                        # check if any of the args to isinstance is a PHI node
+                        iv = func_ir._definitions[arg.name]
+                        assert len(iv) == 1  # SSA!
+                        if isinstance(iv[0], ir.Expr) and iv[0].op == 'phi':
+                            msg = (f'{fn.name}() cannot determine the '
+                                   f'type of variable "{arg.unversioned_name}" '
+                                   'due to a branch.')
+                            raise errors.NumbaTypeError(msg, loc=assign.loc)
+
+                # Only propagate a PHI node if all arguments are the same
+                # constant
+                if isinstance(value, ir.Expr) and value.op == 'phi':
+                    # typemap will return None in case `inc.name` not in typemap
+                    v = [typemap.get(inc.name) for inc in value.incoming_values]
+                    # stop if the elements in `v` do not hold the same value
+                    if v[0] is not None and any([v[0] != vi for vi in v]):
+                        continue
+
+                lit = typemap.get(target.name, None)
+                if lit and isinstance(lit, types.Literal):
+                    # replace assign instruction by ir.Const(lit) iff
+                    # lit is a literal value
+                    rhs = ir.Const(lit.literal_value, assign.loc)
+                    new_assign = ir.Assign(rhs, target, assign.loc)
+
+                    # replace instruction
+                    block.insert_after(new_assign, assign)
+                    block.remove(assign)
+
+                    changed = True
+
+        # reset type inference now we are done with the partial results
+        state.typemap = None
+        state.calltypes = None
+
+        if changed:
+            # Rebuild definitions
+            func_ir._definitions = build_definitions(func_ir.blocks)
+
+        return changed
+
+
+@register_pass(mutates_CFG=True, analysis_only=False)
+class LiteralPropagationSubPipelinePass(FunctionPass):
+    """Implement literal propagation based on partial type inference"""
+    _name = "LiteralPropagation"
+
+    def __init__(self):
+        FunctionPass.__init__(self)
+
+    def run_pass(self, state):
+        # Determine whether to even attempt this pass... if there's no
+        # `isinstance` as a global or as a freevar then just skip.
+
+        found = False
+        func_ir = state.func_ir
+        for blk in func_ir.blocks.values():
+            for asgn in blk.find_insts(ir.Assign):
+                if isinstance(asgn.value, (ir.Global, ir.FreeVar)):
+                    value = asgn.value.value
+                    if value is isinstance or value is hasattr:
+                        found = True
+                        break
+            if found:
+                break
+        if not found:
+            return False
+
+        # run as subpipeline
+        from numba.core.compiler_machinery import PassManager
+        from numba.core.typed_passes import PartialTypeInference
+        pm = PassManager("literal_propagation_subpipeline")
+
+        pm.add_pass(PartialTypeInference, "performs partial type inference")
+        pm.add_pass(PropagateLiterals, "performs propagation of literal values")
+
+        # rewrite consts / dead branch pruning
+        pm.add_pass(RewriteSemanticConstants, "rewrite semantic constants")
+        pm.add_pass(DeadBranchPrune, "dead branch pruning")
+
+        pm.finalize()
+        pm.run(state)
+        return True
+
+    def get_analysis_usage(self, AU):
+        AU.add_required(ReconstructSSA)
+
+
 @register_pass(mutates_CFG=True, analysis_only=False)
 class LiteralUnroll(FunctionPass):
     """Implement the literal_unroll semantics"""
@@ -1432,7 +1671,7 @@ class LiteralUnroll(FunctionPass):
 
     def run_pass(self, state):
         # Determine whether to even attempt this pass... if there's no
-        # `literal_unroll as a global or as a freevar then just skip.
+        # `literal_unroll` as a global or as a freevar then just skip.
         found = False
         func_ir = state.func_ir
         for blk in func_ir.blocks.values():
@@ -1464,6 +1703,10 @@ class LiteralUnroll(FunctionPass):
         pm.add_pass(RewriteSemanticConstants, "rewrite semantic constants")
         # do the unroll
         pm.add_pass(MixedContainerUnroller, "performs mixed container unroll")
+        # rewrite dynamic getitem to static getitem as it's possible some more
+        # getitems will now be statically resolvable
+        pm.add_pass(GenericRewrites, "Generic Rewrites")
+        pm.add_pass(RewriteSemanticConstants, "rewrite semantic constants")
         pm.finalize()
         pm.run(state)
         return True
@@ -1528,3 +1771,43 @@ class ReconstructSSA(FunctionPass):
                 typ = locals_dict[parent]
                 for derived in redefs:
                     locals_dict[derived] = typ
+
+
+@register_pass(mutates_CFG=False, analysis_only=False)
+class RewriteDynamicRaises(FunctionPass):
+    """Replace existing raise statements by dynamic raises in Numba IR.
+    """
+    _name = "Rewrite dynamic raises"
+
+    def __init__(self):
+        FunctionPass.__init__(self)
+
+    def run_pass(self, state):
+        func_ir = state.func_ir
+        changed = False
+
+        for block in func_ir.blocks.values():
+            for raise_ in block.find_insts((ir.Raise, ir.TryRaise)):
+                call_inst = guard(get_definition, func_ir, raise_.exception)
+                if call_inst is None:
+                    continue
+                exc_type = func_ir.infer_constant(call_inst.func.name)
+                exc_args = []
+                for exc_arg in call_inst.args:
+                    try:
+                        const = func_ir.infer_constant(exc_arg)
+                        exc_args.append(const)
+                    except consts.ConstantInferenceError:
+                        exc_args.append(exc_arg)
+                loc = raise_.loc
+
+                cls = {
+                    ir.TryRaise: ir.DynamicTryRaise,
+                    ir.Raise: ir.DynamicRaise,
+                }[type(raise_)]
+
+                dyn_raise = cls(exc_type, tuple(exc_args), loc)
+                block.insert_after(dyn_raise, raise_)
+                block.remove(raise_)
+                changed = True
+        return changed
