@@ -4,16 +4,20 @@ import functools
 import inspect
 import os
 import operator
-import threading
 import timeit
 import math
 import sys
 import traceback
 import weakref
 import warnings
+import threading
+import contextlib
+import json
+import typing as _tp
+from pprint import pformat
+
 from types import ModuleType
 from importlib import import_module
-from collections.abc import Mapping, Sequence
 import numpy as np
 
 from inspect import signature as pysignature # noqa: F401
@@ -22,77 +26,10 @@ from inspect import Parameter as pyParameter # noqa: F401
 
 from numba.core.config import (PYVERSION, MACHINE_BITS, # noqa: F401
                                DEVELOPER_MODE) # noqa: F401
+from numba.core import config
 from numba.core import types
 
-INT_TYPES = (int,)
-longint = int
-get_ident = threading.get_ident
-intern = sys.intern
-file_replace = os.replace
-asbyteint = int
-
-# ------------------------------------------------------------------------------
-# Start: Originally from `numba.six` under the following license
-
-"""Utilities for writing code that runs on Python 2 and 3"""
-
-# Copyright (c) 2010-2015 Benjamin Peterson
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in all
-# copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-# SOFTWARE.
-
-
-def add_metaclass(metaclass):
-    """Class decorator for creating a class with a metaclass."""
-    def wrapper(cls):
-        orig_vars = cls.__dict__.copy()
-        slots = orig_vars.get('__slots__')
-        if slots is not None:
-            if isinstance(slots, str):
-                slots = [slots]
-            for slots_var in slots:
-                orig_vars.pop(slots_var)
-        orig_vars.pop('__dict__', None)
-        orig_vars.pop('__weakref__', None)
-        return metaclass(cls.__name__, cls.__bases__, orig_vars)
-    return wrapper
-
-
-def reraise(tp, value, tb=None):
-    if value is None:
-        value = tp()
-    if value.__traceback__ is not tb:
-        raise value.with_traceback(tb)
-    raise value
-
-
-def iteritems(d, **kw):
-    return iter(d.items(**kw))
-
-
-def itervalues(d, **kw):
-    return iter(d.values(**kw))
-
-
-get_function_globals = operator.attrgetter("__globals__")
-
-# End: Originally from `numba.six` under the following license
-# ------------------------------------------------------------------------------
+from collections.abc import Mapping, Sequence, MutableSet, MutableMapping
 
 
 def erase_traceback(exc_value):
@@ -104,6 +41,23 @@ def erase_traceback(exc_value):
     return exc_value.with_traceback(None)
 
 
+def safe_relpath(path, start=os.curdir):
+    """
+    Produces a "safe" relative path, on windows relpath doesn't work across
+    drives as technically they don't share the same root.
+    See: https://bugs.python.org/issue7195 for details.
+    """
+    # find the drive letters for path and start and if they are not the same
+    # then don't use relpath!
+    drive_letter = lambda x: os.path.splitdrive(os.path.abspath(x))[0]
+    drive_path = drive_letter(path)
+    drive_start = drive_letter(start)
+    if drive_path != drive_start:
+        return os.path.abspath(path)
+    else:
+        return os.path.relpath(path, start=start)
+
+
 # Mapping between operator module functions and the corresponding built-in
 # operators.
 
@@ -129,7 +83,8 @@ BINOPS_TO_OPERATORS = {
     'is': operator.is_,
     'is not': operator.is_not,
     # This one has its args reversed!
-    'in': operator.contains
+    'in': operator.contains,
+    '@': operator.matmul,
 }
 
 INPLACE_BINOPS_TO_OPERATORS = {
@@ -145,8 +100,14 @@ INPLACE_BINOPS_TO_OPERATORS = {
     '^=': operator.ixor,
     '<<=': operator.ilshift,
     '>>=': operator.irshift,
+    '@=': operator.imatmul,
 }
 
+
+ALL_BINOPS_TO_OPERATORS = {**BINOPS_TO_OPERATORS,
+                           **INPLACE_BINOPS_TO_OPERATORS}
+
+
 UNARY_BUITINS_TO_OPERATORS = {
     '+': operator.pos,
     '-': operator.neg,
@@ -198,9 +159,6 @@ OPERATORS_TO_BUILTINS = {
     operator.truth: 'is_true',
 }
 
-BINOPS_TO_OPERATORS['@'] = operator.matmul
-INPLACE_BINOPS_TO_OPERATORS['@='] = operator.imatmul
-
 
 _shutting_down = False
 
@@ -226,12 +184,75 @@ def shutting_down(globals=globals):
 # which atexit is True. Some of these finalizers may call shutting_down() to
 # check whether the interpreter is shutting down. For this to behave correctly,
 # we need to make sure that _at_shutdown is called before the finalizer exit
-# function. Since atexit operates as a LIFO stack, we first contruct a dummy
+# function. Since atexit operates as a LIFO stack, we first construct a dummy
 # finalizer then register atexit to ensure this ordering.
 weakref.finalize(lambda: None, lambda: None)
 atexit.register(_at_shutdown)
 
 
+class ThreadLocalStack:
+    """A TLS stack container.
+
+    Uses the BORG pattern and stores states in threadlocal storage.
+    """
+    _tls = threading.local()
+    stack_name: str
+    _registered = {}
+
+    def __init_subclass__(cls, *, stack_name, **kwargs):
+        super().__init_subclass__(**kwargs)
+        # Register stack_name mapping to the new subclass
+        assert stack_name not in cls._registered, \
+            f"stack_name: '{stack_name}' already in use"
+        cls.stack_name = stack_name
+        cls._registered[stack_name] = cls
+
+    def __init__(self):
+        # This class must not be used directly.
+        assert type(self) is not ThreadLocalStack
+        tls = self._tls
+        attr = f"stack_{self.stack_name}"
+        try:
+            tls_stack = getattr(tls, attr)
+        except AttributeError:
+            tls_stack = list()
+            setattr(tls, attr, tls_stack)
+
+        self._stack = tls_stack
+
+    def push(self, state):
+        """Push to the stack
+        """
+        self._stack.append(state)
+
+    def pop(self):
+        """Pop from the stack
+        """
+        return self._stack.pop()
+
+    def top(self):
+        """Get the top item on the stack.
+
+        Raises IndexError if the stack is empty. Users should check the size
+        of the stack beforehand.
+        """
+        return self._stack[-1]
+
+    def __len__(self):
+        return len(self._stack)
+
+    @contextlib.contextmanager
+    def enter(self, state):
+        """A contextmanager that pushes ``state`` for the duration of the
+        context.
+        """
+        self.push(state)
+        try:
+            yield
+        finally:
+            self.pop()
+
+
 class ConfigOptions(object):
     OPTIONS = {}
 
@@ -282,7 +303,101 @@ class ConfigOptions(object):
         return hash(tuple(sorted(self._values.items())))
 
 
-class SortedMap(Mapping):
+def order_by_target_specificity(target, templates, fnkey=''):
+    """This orders the given templates from most to least specific against the
+    current "target". "fnkey" is an indicative typing key for use in the
+    exception message in the case that there's no usable templates for the
+    current "target".
+    """
+    # No templates... return early!
+    if templates == []:
+        return []
+
+    from numba.core.target_extension import target_registry
+
+    # fish out templates that are specific to the target if a target is
+    # specified
+    DEFAULT_TARGET = 'generic'
+    usable = []
+    for ix, temp_cls in enumerate(templates):
+        # ? Need to do something about this next line
+        md = getattr(temp_cls, "metadata", {})
+        hw = md.get('target', DEFAULT_TARGET)
+        if hw is not None:
+            hw_clazz = target_registry[hw]
+            if target.inherits_from(hw_clazz):
+                usable.append((temp_cls, hw_clazz, ix))
+
+    # sort templates based on target specificity
+    def key(x):
+        return target.__mro__.index(x[1])
+    order = [x[0] for x in sorted(usable, key=key)]
+
+    if not order:
+        msg = (f"Function resolution cannot find any matches for function "
+               f"'{fnkey}' for the current target: '{target}'.")
+        from numba.core.errors import UnsupportedError
+        raise UnsupportedError(msg)
+
+    return order
+
+
+T = _tp.TypeVar('T')
+
+
+class OrderedSet(MutableSet[T]):
+
+    def __init__(self, iterable: _tp.Iterable[T] = ()):
+        # Just uses a dictionary under-the-hood to maintain insertion order.
+        self._data = dict.fromkeys(iterable, None)
+
+    def __contains__(self, key):
+        return key in self._data
+
+    def __iter__(self):
+        return iter(self._data)
+
+    def __len__(self):
+        return len(self._data)
+
+    def add(self, item):
+        self._data[item] = None
+
+    def discard(self, item):
+        self._data.pop(item, None)
+
+
+class MutableSortedSet(MutableSet[T], _tp.Generic[T]):
+    """Mutable Sorted Set
+    """
+
+    def __init__(self, values: _tp.Iterable[T] = ()):
+        self._values = set(values)
+
+    def __len__(self):
+        return len(self._values)
+
+    def __iter__(self):
+        return iter(k for k in sorted(self._values))
+
+    def __contains__(self, x: T) -> bool:
+        return self._values.__contains__(x)
+
+    def add(self, x: T):
+        return self._values.add(x)
+
+    def discard(self, value: T):
+        self._values.discard(value)
+
+    def update(self, values):
+        self._values.update(values)
+
+
+Tk = _tp.TypeVar('Tk')
+Tv = _tp.TypeVar('Tv')
+
+
+class SortedMap(Mapping[Tk, Tv], _tp.Generic[Tk, Tv]):
     """Immutable
     """
 
@@ -304,6 +419,28 @@ class SortedMap(Mapping):
         return iter(k for k, v in self._values)
 
 
+class MutableSortedMap(MutableMapping[Tk, Tv], _tp.Generic[Tk, Tv]):
+    def __init__(self, dct=None):
+        if dct is None:
+            dct = {}
+        self._dct: dict[Tk, Tv] = dct
+
+    def __getitem__(self, k: Tk) -> Tv:
+        return self._dct[k]
+
+    def __setitem__(self, k: Tk, v: Tv):
+        self._dct[k] = v
+
+    def __delitem__(self, k: Tk):
+        del self._dct[k]
+
+    def __len__(self) -> int:
+        return len(self._dct)
+
+    def __iter__(self) -> int:
+        return iter(k for k in sorted(self._dct))
+
+
 class UniqueDict(dict):
     def __setitem__(self, key, value):
         if key in self:
@@ -311,28 +448,6 @@ class UniqueDict(dict):
         super(UniqueDict, self).__setitem__(key, value)
 
 
-# Django's cached_property
-# see https://docs.djangoproject.com/en/dev/ref/utils/#django.utils.functional.cached_property    # noqa: E501
-
-class cached_property(object):
-    """
-    Decorator that converts a method with a single self argument into a
-    property cached on the instance.
-
-    Optional ``name`` argument allows you to make cached properties of other
-    methods. (e.g.  url = cached_property(get_absolute_url, name='url') )
-    """
-    def __init__(self, func, name=None):
-        self.func = func
-        self.name = name or func.__name__
-
-    def __get__(self, instance, type=None):
-        if instance is None:
-            return self
-        res = instance.__dict__[self.name] = self.func(instance)
-        return res
-
-
 def runonce(fn):
     @functools.wraps(fn)
     def inner():
@@ -350,7 +465,7 @@ def bit_length(intval):
     """
     Return the number of bits necessary to represent integer `intval`.
     """
-    assert isinstance(intval, INT_TYPES)
+    assert isinstance(intval, int)
     if intval >= 0:
         return len(bin(intval)) - 2
     else:
@@ -421,9 +536,6 @@ def benchmark(func, maxsec=1):
     return BenchmarkResult(func, records, number)
 
 
-RANGE_ITER_OBJECTS = (builtins.range,)
-
-
 # A dummy module for dynamically-generated functions
 _dynamic_modname = '<dynamic>'
 _dynamic_module = ModuleType(_dynamic_modname)
@@ -576,6 +688,9 @@ class _RedirectSubpackage(ModuleType):
         old_module = old_module_locals['__name__']
         super().__init__(old_module)
 
+        self.__old_module_states = {}
+        self.__new_module = new_module
+
         new_mod_obj = import_module(new_module)
 
         # Map all sub-modules over
@@ -591,4 +706,70 @@ class _RedirectSubpackage(ModuleType):
         # copy across dunders so that package imports work too
         for attr, value in old_module_locals.items():
             if attr.startswith('__') and attr.endswith('__'):
-                setattr(self, attr, value)
+                if attr != "__builtins__":
+                    setattr(self, attr, value)
+                    self.__old_module_states[attr] = value
+
+    def __reduce__(self):
+        args = (self.__old_module_states, self.__new_module)
+        return _RedirectSubpackage, args
+
+
+def get_hashable_key(value):
+    """
+        Given a value, returns a key that can be used
+        as a hash. If the value is hashable, we return
+        the value, otherwise we return id(value).
+
+        See discussion in gh #6957
+    """
+    try:
+        hash(value)
+    except TypeError:
+        return id(value)
+    else:
+        return value
+
+
+class threadsafe_cached_property(functools.cached_property):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._lock = threading.RLock()
+
+    def __get__(self, *args, **kwargs):
+        with self._lock:
+            return super().__get__(*args, **kwargs)
+
+
+def dump_llvm(fndesc, module):
+    print(("LLVM DUMP %s" % fndesc).center(80, '-'))
+    if config.HIGHLIGHT_DUMPS:
+        try:
+            from pygments import highlight
+            from pygments.lexers import LlvmLexer as lexer
+            from pygments.formatters import Terminal256Formatter
+            from numba.misc.dump_style import by_colorscheme
+            print(highlight(module.__repr__(), lexer(),
+                            Terminal256Formatter( style=by_colorscheme())))
+        except ImportError:
+            msg = "Please install pygments to see highlighted dumps"
+            raise ValueError(msg)
+    else:
+        print(module)
+    print('=' * 80)
+
+
+class _lazy_pformat(object):
+    def __init__(self, *args, **kwargs):
+        self.args = args
+        self.kwargs = kwargs
+
+    def __str__(self):
+        return pformat(*self.args, **self.kwargs)
+
+
+class _LazyJSONEncoder(json.JSONEncoder):
+    def default(self, obj):
+        if isinstance(obj, _lazy_pformat):
+            return str(obj)
+        return super().default(obj)
