@@ -1,7 +1,6 @@
-from __future__ import absolute_import, print_function, division
 import numpy as np
 from numba import cuda, int32, float32
-from numba.cuda.testing import unittest, SerialMixin
+from numba.cuda.testing import unittest, CUDATestCase
 
 N = 100
 
@@ -28,7 +27,7 @@ def coop_smem2d(ary):
     ary[i, j] = sm[i, j]
 
 
-class TestCudaTestGlobal(SerialMixin, unittest.TestCase):
+class TestCudaTestGlobal(CUDATestCase):
     def test_global_int_const(self):
         """Test simple_smem
         """
