@@ -55,11 +55,6 @@ class LibraryLookupBase(SerialMixin, unittest.TestCase):
     def do_terminate():
         return False, None
 
-    def check_NUMBAPRO_warning(self, warns):
-        msg = "Environment variables with the 'NUMBAPRO' prefix are deprecated"
-        self.assertEqual(len(warns), 1)
-        self.assertIn(msg, str(warns[0].message))
-
 
 def remove_env(name):
     try:
@@ -108,12 +103,6 @@ class TestLibDeviceLookUp(LibraryLookupBase):
         self.assertEqual(info, os.path.join('mycudahome', 'nvvm', 'libdevice'))
         self.assertFalse(warns)
 
-        # Check that NUMBAPRO_LIBDEVICE override *warns*
-        by, info, warns = self.remote_do(self.do_set_libdevice)
-        self.check_NUMBAPRO_warning(warns)
-        # clear
-        self.remote_do(self.do_clear_envs)
-
         if get_system_ctk() is None:
             # Fake remove conda environment so no cudatoolkit is available
             by, info, warns = self.remote_do(self.do_clear_envs)
@@ -130,8 +119,6 @@ class TestLibDeviceLookUp(LibraryLookupBase):
     def do_clear_envs():
         remove_env('CUDA_HOME')
         remove_env('CUDA_PATH')
-        remove_env('NUMBAPRO_CUDALIB')
-        remove_env('NUMBAPRO_LIBDEVICE')
         return True, _get_libdevice_path_decision()
 
     @staticmethod
@@ -140,11 +127,6 @@ class TestLibDeviceLookUp(LibraryLookupBase):
         _fake_non_conda_env()
         return True, _get_libdevice_path_decision()
 
-    @staticmethod
-    def do_set_libdevice():
-        os.environ['NUMBAPRO_LIBDEVICE'] = 'nbp_libdevice'
-        return True, _get_libdevice_path_decision()
-
 
 @skip_on_cudasim('Library detection unsupported in the simulator')
 @unittest.skipUnless(has_mp_get_context, 'mp.get_context not available')
@@ -170,16 +152,6 @@ class TestNvvmLookUp(LibraryLookupBase):
         else:
             self.assertEqual(info, os.path.join('mycudahome', 'nvvm', 'lib64'))
 
-        # Check that NUMBAPRO_CUDALIB override warns
-        self.remote_do(self.do_clear_envs)
-        by, info, warns = self.remote_do(self.do_set_cuda_lib)
-        self.check_NUMBAPRO_warning(warns)
-
-        # Check that NUMBAPRO_NVVM override warns
-        self.remote_do(self.do_clear_envs)
-        by, info, warns = self.remote_do(self.do_set_nvvm)
-        self.check_NUMBAPRO_warning(warns)
-
         if get_system_ctk() is None:
             # Fake remove conda environment so no cudatoolkit is available
             by, info, warns = self.remote_do(self.do_clear_envs)
@@ -196,8 +168,6 @@ class TestNvvmLookUp(LibraryLookupBase):
     def do_clear_envs():
         remove_env('CUDA_HOME')
         remove_env('CUDA_PATH')
-        remove_env('NUMBAPRO_CUDALIB')
-        remove_env('NUMBAPRO_NVVM')
         return True, _get_nvvm_path_decision()
 
     @staticmethod
@@ -206,16 +176,6 @@ class TestNvvmLookUp(LibraryLookupBase):
         _fake_non_conda_env()
         return True, _get_nvvm_path_decision()
 
-    @staticmethod
-    def do_set_cuda_lib():
-        os.environ['NUMBAPRO_CUDALIB'] = 'nbp_cudalib'
-        return True, _get_nvvm_path_decision()
-
-    @staticmethod
-    def do_set_nvvm():
-        os.environ['NUMBAPRO_NVVM'] = 'nbp_nvvm'
-        return True, _get_nvvm_path_decision()
-
 
 @skip_on_cudasim('Library detection unsupported in the simulator')
 @unittest.skipUnless(has_mp_get_context, 'mp.get_context not available')
@@ -231,10 +191,6 @@ class TestCudaLibLookUp(LibraryLookupBase):
             self.assertIsNone(info)
         self.assertFalse(warns)
 
-        # Check that NUMBAPRO_CUDALIB override warns
-        by, info, warns = self.remote_do(self.do_set_cuda_lib)
-        self.check_NUMBAPRO_warning(warns)
-
         # Check that CUDA_HOME works by removing conda-env
         self.remote_do(self.do_clear_envs)
         by, info, warns = self.remote_do(self.do_set_cuda_home)
@@ -262,7 +218,6 @@ class TestCudaLibLookUp(LibraryLookupBase):
     def do_clear_envs():
         remove_env('CUDA_HOME')
         remove_env('CUDA_PATH')
-        remove_env('NUMBAPRO_CUDALIB')
         return True, _get_cudalib_dir_path_decision()
 
     @staticmethod
@@ -271,14 +226,13 @@ class TestCudaLibLookUp(LibraryLookupBase):
         _fake_non_conda_env()
         return True, _get_cudalib_dir_path_decision()
 
-    @staticmethod
-    def do_set_cuda_lib():
-        os.environ['NUMBAPRO_CUDALIB'] = 'nbp_cudalib'
-        return True, _get_cudalib_dir_path_decision()
-
 
 def _fake_non_conda_env():
     """
     Monkeypatch sys.prefix to hide the fact we are in a conda-env
     """
     sys.prefix = ''
+
+
+if __name__ == '__main__':
+    unittest.main()
