@@ -4,9 +4,10 @@ import re
 
 import numpy as np
 
-from numba.core import errors, types
+from numba.core import errors, types, config
 from numba.core.typing.templates import signature
-
+from numba.np import npdatetime_helpers
+from numba.core.errors import TypingError
 
 # re-export
 from numba.core.cgutils import is_nonelike   # noqa: F401
@@ -15,26 +16,49 @@ from numba.core.cgutils import is_nonelike   # noqa: F401
 numpy_version = tuple(map(int, np.__version__.split('.')[:2]))
 
 
-FROM_DTYPE = {
-    np.dtype('bool'): types.boolean,
-    np.dtype('int8'): types.int8,
-    np.dtype('int16'): types.int16,
-    np.dtype('int32'): types.int32,
-    np.dtype('int64'): types.int64,
+if config.USE_LEGACY_TYPE_SYSTEM:
+    FROM_DTYPE = {
+        np.dtype('bool'): types.boolean,
+        np.dtype('int8'): types.int8,
+        np.dtype('int16'): types.int16,
+        np.dtype('int32'): types.int32,
+        np.dtype('int64'): types.int64,
 
-    np.dtype('uint8'): types.uint8,
-    np.dtype('uint16'): types.uint16,
-    np.dtype('uint32'): types.uint32,
-    np.dtype('uint64'): types.uint64,
+        np.dtype('uint8'): types.uint8,
+        np.dtype('uint16'): types.uint16,
+        np.dtype('uint32'): types.uint32,
+        np.dtype('uint64'): types.uint64,
 
-    np.dtype('float32'): types.float32,
-    np.dtype('float64'): types.float64,
+        np.dtype('float32'): types.float32,
+        np.dtype('float64'): types.float64,
+        np.dtype('float16'): types.float16,
+        np.dtype('complex64'): types.complex64,
+        np.dtype('complex128'): types.complex128,
 
-    np.dtype('complex64'): types.complex64,
-    np.dtype('complex128'): types.complex128,
+        np.dtype(object): types.pyobject,
+    }
+else:
+    FROM_DTYPE = {
+        np.dtype('bool'): types.np_bool_,
+        np.dtype('int8'): types.np_int8,
+        np.dtype('int16'): types.np_int16,
+        np.dtype('int32'): types.np_int32,
+        np.dtype('int64'): types.np_int64,
+
+        np.dtype('uint8'): types.np_uint8,
+        np.dtype('uint16'): types.np_uint16,
+        np.dtype('uint32'): types.np_uint32,
+        np.dtype('uint64'): types.np_uint64,
+
+        np.dtype('float32'): types.np_float32,
+        np.dtype('float64'): types.np_float64,
+        np.dtype('float16'): types.np_float16,
+        np.dtype('complex64'): types.np_complex64,
+        np.dtype('complex128'): types.np_complex128,
+
+        np.dtype(object): types.pyobject,
+    }
 
-    np.dtype(object): types.pyobject,
-}
 
 re_typestr = re.compile(r'[<>=\|]([a-z])(\d+)?$', re.I)
 re_datetimestr = re.compile(r'[<>=\|]([mM])8?(\[([a-z]+)\])?$', re.I)
@@ -45,14 +69,14 @@ sizeof_unicode_char = np.dtype('U1').itemsize
 def _from_str_dtype(dtype):
     m = re_typestr.match(dtype.str)
     if not m:
-        raise NotImplementedError(dtype)
+        raise errors.NumbaNotImplementedError(dtype)
     groups = m.groups()
     typecode = groups[0]
     if typecode == 'U':
         # unicode
         if dtype.byteorder not in '=|':
-            raise NotImplementedError("Does not support non-native "
-                                      "byteorder")
+            raise errors.NumbaNotImplementedError("Does not support non-native "
+                                                  "byteorder")
         count = dtype.itemsize // sizeof_unicode_char
         assert count == int(groups[1]), "Unicode char size mismatch"
         return types.UnicodeCharSeq(count)
@@ -64,13 +88,13 @@ def _from_str_dtype(dtype):
         return types.CharSeq(count)
 
     else:
-        raise NotImplementedError(dtype)
+        raise errors.NumbaNotImplementedError(dtype)
 
 
 def _from_datetime_dtype(dtype):
     m = re_datetimestr.match(dtype.str)
     if not m:
-        raise NotImplementedError(dtype)
+        raise errors.NumbaNotImplementedError(dtype)
     groups = m.groups()
     typecode = groups[0]
     unit = groups[2] or ''
@@ -79,15 +103,15 @@ def _from_datetime_dtype(dtype):
     elif typecode == 'M':
         return types.NPDatetime(unit)
     else:
-        raise NotImplementedError(dtype)
+        raise errors.NumbaNotImplementedError(dtype)
 
 
 def from_dtype(dtype):
     """
     Return a Numba Type instance corresponding to the given Numpy *dtype*.
-    NotImplementedError is raised on unsupported Numpy dtypes.
+    NumbaNotImplementedError is raised on unsupported Numpy dtypes.
     """
-    if type(dtype) == type and issubclass(dtype, np.generic):
+    if type(dtype) is type and issubclass(dtype, np.generic):
         dtype = np.dtype(dtype)
     elif getattr(dtype, "fields", None) is not None:
         return from_struct_dtype(dtype)
@@ -95,17 +119,22 @@ def from_dtype(dtype):
     try:
         return FROM_DTYPE[dtype]
     except KeyError:
-        char = dtype.char
+        pass
 
+    try:
+        char = dtype.char
+    except AttributeError:
+        pass
+    else:
         if char in 'SU':
             return _from_str_dtype(dtype)
         if char in 'mM':
             return _from_datetime_dtype(dtype)
-        if char in 'V':
+        if char in 'V' and dtype.subdtype is not None:
             subtype = from_dtype(dtype.subdtype[0])
             return types.NestedArray(subtype, dtype.shape)
 
-    raise NotImplementedError(dtype)
+    raise errors.NumbaNotImplementedError(dtype)
 
 
 _as_dtype_letters = {
@@ -124,7 +153,7 @@ def as_dtype(nbtype):
     nbtype = types.unliteral(nbtype)
     if isinstance(nbtype, (types.Complex, types.Integer, types.Float)):
         return np.dtype(str(nbtype))
-    if nbtype is types.bool_:
+    if isinstance(nbtype, (types.Boolean)):
         return np.dtype('?')
     if isinstance(nbtype, (types.NPDatetime, types.NPTimedelta)):
         letter = _as_dtype_letters[type(nbtype)]
@@ -148,8 +177,9 @@ def as_dtype(nbtype):
         return np.dtype(spec)
     if isinstance(nbtype, types.PyObject):
         return np.dtype(object)
-    raise NotImplementedError("%r cannot be represented as a Numpy dtype"
-                              % (nbtype,))
+
+    msg = f"{nbtype} cannot be represented as a NumPy dtype"
+    raise errors.NumbaNotImplementedError(msg)
 
 
 def as_struct_dtype(rec):
@@ -203,8 +233,8 @@ def map_arrayscalar_type(val):
         try:
             dtype = np.dtype(type(val))
         except TypeError:
-            raise NotImplementedError("no corresponding numpy dtype "
-                                      "for %r" % type(val))
+            raise errors.NumbaNotImplementedError("no corresponding numpy "
+                                                  "dtype for %r" % type(val))
     return from_dtype(dtype)
 
 
@@ -279,6 +309,7 @@ def supported_ufunc_loop(ufunc, loop):
     legacy and when implementing new ufuncs the ufunc_db should be preferred,
     as it allows for a more fine-grained incremental support.
     """
+    # NOTE: Assuming ufunc for the CPUContext
     from numba.np import ufunc_db
     loop_sig = loop.ufunc_sig
     try:
@@ -360,15 +391,15 @@ def ufunc_find_matching_loop(ufunc, arg_types):
     # Separate logical input from explicit output arguments
     input_types = arg_types[:ufunc.nin]
     output_types = arg_types[ufunc.nin:]
-    assert(len(input_types) == ufunc.nin)
+    assert (len(input_types) == ufunc.nin)
 
     try:
         np_input_types = [as_dtype(x) for x in input_types]
-    except NotImplementedError:
+    except errors.NumbaNotImplementedError:
         return None
     try:
         np_output_types = [as_dtype(x) for x in output_types]
-    except NotImplementedError:
+    except errors.NumbaNotImplementedError:
         return None
 
     # Whether the inputs are mixed integer / floating-point
@@ -390,7 +421,7 @@ def ufunc_find_matching_loop(ufunc, arg_types):
                   for letter in ufunc_letters[len(numba_types):]]
         return types
 
-    def set_output_dt_units(inputs, outputs, ufunc_inputs):
+    def set_output_dt_units(inputs, outputs, ufunc_inputs, ufunc_name):
         """
         Sets the output unit of a datetime type based on the input units
 
@@ -424,6 +455,23 @@ def ufunc_find_matching_loop(ufunc, arg_types):
                     new_outputs.append(out)
             return new_outputs
 
+        def make_datetime_specific(outputs, dt_unit, td_unit):
+            new_outputs = []
+            for out in outputs:
+                if isinstance(out, types.NPDatetime) and out.unit == "":
+                    unit = npdatetime_helpers.combine_datetime_timedelta_units(
+                        dt_unit, td_unit)
+                    if unit is None:
+                        raise TypingError(f"ufunc '{ufunc_name}' is not " +
+                                          "supported between " +
+                                          f"datetime64[{dt_unit}] " +
+                                          f"and timedelta64[{td_unit}]"
+                                          )
+                    new_outputs.append(types.NPDatetime(unit))
+                else:
+                    new_outputs.append(out)
+            return new_outputs
+
         if ufunc_inputs == 'mm':
             if all(inp.unit == inputs[0].unit for inp in inputs):
                 # Case with operation on same units. Operations on different
@@ -434,6 +482,20 @@ def ufunc_find_matching_loop(ufunc, arg_types):
             else:
                 return outputs
             return new_outputs
+        elif ufunc_inputs == 'mM':
+            # case where the left operand has timedelta type
+            # and the right operand has datetime
+            td_unit = inputs[0].unit
+            dt_unit = inputs[1].unit
+            return make_datetime_specific(outputs, dt_unit, td_unit)
+
+        elif ufunc_inputs == 'Mm':
+            # case where the right operand has timedelta type
+            # and the left operand has datetime
+            dt_unit = inputs[0].unit
+            td_unit = inputs[1].unit
+            return make_datetime_specific(outputs, dt_unit, td_unit)
+
         elif ufunc_inputs[0] == 'm':
             # case where the left operand has timedelta type
             unit = inputs[0].unit
@@ -448,7 +510,11 @@ def ufunc_find_matching_loop(ufunc, arg_types):
 
     for candidate in ufunc.types:
         ufunc_inputs = candidate[:ufunc.nin]
-        ufunc_outputs = candidate[-ufunc.nout:]
+        ufunc_outputs = candidate[-ufunc.nout:] if ufunc.nout else []
+
+        if 'e' in ufunc_inputs:
+            # Skip float16 arrays since we don't have implementation for them
+            continue
         if 'O' in ufunc_inputs:
             # Skip object arrays
             continue
@@ -481,12 +547,14 @@ def ufunc_find_matching_loop(ufunc, arg_types):
             try:
                 inputs = choose_types(input_types, ufunc_inputs)
                 outputs = choose_types(output_types, ufunc_outputs)
-                # if the left operand or both are timedeltas, then the output
-                # units need to be determined.
-                if ufunc_inputs[0] == 'm':
-                    outputs = set_output_dt_units(inputs, outputs, ufunc_inputs)
+                # if the left operand or both are timedeltas, or the first
+                # argument is datetime and the second argument is timedelta,
+                # then the output units need to be determined.
+                if ufunc_inputs[0] == 'm' or ufunc_inputs == 'Mm':
+                    outputs = set_output_dt_units(inputs, outputs,
+                                                  ufunc_inputs, ufunc.__name__)
 
-            except NotImplementedError:
+            except errors.NumbaNotImplementedError:
                 # One of the selected dtypes isn't supported by Numba
                 # (e.g. float16), try other candidates
                 continue
@@ -504,7 +572,8 @@ def from_struct_dtype(dtype):
     """Convert a NumPy structured dtype to Numba Record type
     """
     if dtype.hasobject:
-        raise TypeError("Do not support dtype containing object")
+        msg = "dtypes that contain object are not supported."
+        raise errors.NumbaNotImplementedError(msg)
 
     fields = []
     for name, info in dtype.fields.items():
@@ -577,7 +646,7 @@ def carray(ptr, shape, dtype=None):
     else:
         raise TypeError("expected a ctypes pointer, got %r" % (ptr,))
 
-    nbytes = dtype.itemsize * np.product(shape, dtype=np.intp)
+    nbytes = dtype.itemsize * np.prod(shape, dtype=np.intp)
     return _get_array_from_ptr(p, nbytes, dtype).reshape(shape)
 
 
@@ -667,3 +736,61 @@ def type_can_asarray(arr):
           types.Number, types.Boolean, types.containers.ListType)
 
     return isinstance(arr, ok)
+
+
+def type_is_scalar(typ):
+    """ Returns True if the type of 'typ' is a scalar type, according to
+    NumPy rules. False otherwise.
+    https://numpy.org/doc/stable/reference/arrays.scalars.html#built-in-scalar-types
+    """
+
+    ok = (types.Boolean, types.Number, types.UnicodeType, types.StringLiteral,
+          types.NPTimedelta, types.NPDatetime)
+    return isinstance(typ, ok)
+
+
+def check_is_integer(v, name):
+    """Raises TypingError if the value is not an integer."""
+    if not isinstance(v, (int, types.Integer)):
+        raise TypingError('{} must be an integer'.format(name))
+
+
+def lt_floats(a, b):
+    # Adapted from NumPy commit 717c7acf which introduced the behavior of
+    # putting NaNs at the end.
+    # The code is later moved to numpy/core/src/npysort/npysort_common.h
+    # This info is gathered as of NumPy commit d8c09c50
+    return a < b or (np.isnan(b) and not np.isnan(a))
+
+
+def lt_complex(a, b):
+    if np.isnan(a.real):
+        if np.isnan(b.real):
+            if np.isnan(a.imag):
+                return False
+            else:
+                if np.isnan(b.imag):
+                    return True
+                else:
+                    return a.imag < b.imag
+        else:
+            return False
+
+    else:
+        if np.isnan(b.real):
+            return True
+        else:
+            if np.isnan(a.imag):
+                if np.isnan(b.imag):
+                    return a.real < b.real
+                else:
+                    return False
+            else:
+                if np.isnan(b.imag):
+                    return True
+                else:
+                    if a.real < b.real:
+                        return True
+                    elif a.real == b.real:
+                        return a.imag < b.imag
+                    return False
