@@ -3,6 +3,7 @@ import cmath
 import numpy as np
 
 from numba import float32
+from numba.types import unicode_type, i8
 from numba.pycc import CC, exportmany, export
 from numba.tests.support import has_blas
 from numba import typed
@@ -115,3 +116,19 @@ def dict_usecase(arr):
     for k, v in d.items():
         out[k] = k * v
     return out
+
+# checks for issue #6386
+@cc_nrt.export('internal_str_dict', i8(unicode_type))
+def internal_str_dict(x):
+    d = typed.Dict.empty(unicode_type,i8)
+    if(x not in d):
+        d[x] = len(d)
+    return len(d)
+
+@cc_nrt.export('hash_str', i8(unicode_type))
+def internal_str_dict(x):
+    return hash(x)
+
+@cc_nrt.export('hash_literal_str_A', i8())
+def internal_str_dict():
+    return hash("A")
