@@ -2,10 +2,9 @@ from itertools import product, combinations_with_replacement
 
 import numpy as np
 
-from numba import jit, typeof
-from numba.core.compiler import compile_isolated
+from numba import jit, njit, typeof
 from numba.np.numpy_support import numpy_version
-from numba.tests.support import TestCase, MemoryLeakMixin, tag
+from numba.tests.support import TestCase, MemoryLeakMixin, tag, skip_if_numpy_2
 import unittest
 
 
@@ -75,12 +74,18 @@ def array_min(arr):
 def array_min_global(arr):
     return np.min(arr)
 
+def array_amin(arr):
+    return np.amin(arr)
+
 def array_max(arr):
     return arr.max()
 
 def array_max_global(arr):
     return np.max(arr)
 
+def array_amax(arr):
+    return np.amax(arr)
+
 def array_argmin(arr):
     return arr.argmin()
 
@@ -176,10 +181,9 @@ def full_test_arrays(dtype):
     return array_list
 
 def run_comparative(compare_func, test_array):
-    arrty = typeof(test_array)
-    cres = compile_isolated(compare_func, [arrty])
+    cfunc = njit(compare_func)
     numpy_result = compare_func(test_array)
-    numba_result = cres.entry_point(test_array)
+    numba_result = cfunc(test_array)
 
     return numpy_result, numba_result
 
@@ -458,17 +462,6 @@ class TestArrayReductions(MemoryLeakMixin, TestCase):
         q = np.array(1)
         _check(a, q)
 
-        if numpy_version < (1, 20):
-            # NumPy 1.20+ rewrites the interpolation part of percentile/quantile
-            # to use np.subtract which doesn't support bools.
-            a = True
-            q = False
-            _check(a, q)
-
-            a = np.array([False, True, True])
-            q = a
-            _check(a, q)
-
         a = 5
         q = q_upper_bound / 2
         _check(a, q)
@@ -559,9 +552,7 @@ class TestArrayReductions(MemoryLeakMixin, TestCase):
         self.assertEqual(arrty.ndim, 1)
         self.assertEqual(arrty.layout, 'C')
 
-        cres = compile_isolated(array_sum_global, [arrty])
-        cfunc = cres.entry_point
-
+        cfunc = njit((arrty,),)(array_sum_global)
         self.assertEqual(np.sum(arr), cfunc(arr))
 
     def test_array_prod_int_1d(self):
@@ -570,9 +561,7 @@ class TestArrayReductions(MemoryLeakMixin, TestCase):
         self.assertEqual(arrty.ndim, 1)
         self.assertEqual(arrty.layout, 'C')
 
-        cres = compile_isolated(array_prod, [arrty])
-        cfunc = cres.entry_point
-
+        cfunc = njit((arrty,))(array_prod)
         self.assertEqual(arr.prod(), cfunc(arr))
 
     def test_array_prod_float_1d(self):
@@ -581,9 +570,7 @@ class TestArrayReductions(MemoryLeakMixin, TestCase):
         self.assertEqual(arrty.ndim, 1)
         self.assertEqual(arrty.layout, 'C')
 
-        cres = compile_isolated(array_prod, [arrty])
-        cfunc = cres.entry_point
-
+        cfunc = njit((arrty,))(array_prod)
         np.testing.assert_allclose(arr.prod(), cfunc(arr))
 
     def test_array_prod_global(self):
@@ -592,9 +579,7 @@ class TestArrayReductions(MemoryLeakMixin, TestCase):
         self.assertEqual(arrty.ndim, 1)
         self.assertEqual(arrty.layout, 'C')
 
-        cres = compile_isolated(array_prod_global, [arrty])
-        cfunc = cres.entry_point
-
+        cfunc = njit((arrty,))(array_prod_global)
         np.testing.assert_allclose(np.prod(arr), cfunc(arr))
 
     def check_cumulative(self, pyfunc):
@@ -686,11 +671,6 @@ class TestArrayReductions(MemoryLeakMixin, TestCase):
         np.random.shuffle(arr)
         self.assertPreciseEqual(cfunc(arr), pyfunc(arr))
         # Test with a NaT
-        if numpy_version != (1, 21) and 'median' not in pyfunc.__name__:
-            # There's problems with NaT handling in "median" on at least NumPy
-            # 1.21.{3, 4}. See https://github.com/numpy/numpy/issues/20376
-            arr[arr.size // 2] = 'NaT'
-            self.assertPreciseEqual(cfunc(arr), pyfunc(arr))
         if 'median' not in pyfunc.__name__:
             # Test with (val, NaT)^N (and with the random NaT from above)
             # use a loop, there's some weird thing/bug with arr[1::2] = 'NaT'
@@ -821,6 +801,7 @@ class TestArrayReductions(MemoryLeakMixin, TestCase):
         for a in a_variations():
             check(a)
 
+    @skip_if_numpy_2
     def test_ptp_method(self):
         # checks wiring of np.ndarray.ptp() only, `np.ptp` test above checks
         # the actual alg
@@ -1000,6 +981,8 @@ class TestArrayReductions(MemoryLeakMixin, TestCase):
         assert_raises(arr1d, -2)
         assert_raises(arr2d, -3)
         assert_raises(arr2d, 2)
+        # Exceptions leak references
+        self.disable_leak_check()
 
     def test_argmax_axis_must_be_integer(self):
         arr = np.arange(6)
@@ -1073,6 +1056,9 @@ class TestArrayReductions(MemoryLeakMixin, TestCase):
         assert_raises(arr2d, -3)
         assert_raises(arr2d, 2)
 
+        # Exceptions leak references
+        self.disable_leak_check()
+
     def test_argmin_axis_must_be_integer(self):
         arr = np.arange(6)
 
@@ -1116,6 +1102,7 @@ class TestArrayReductions(MemoryLeakMixin, TestCase):
                            array_all, array_all_global,
                            array_any, array_any_global,
                            array_min, array_min_global,
+                           array_amax, array_amin,
                            array_max, array_max_global,
                            array_nanmax, array_nanmin,
                            array_nansum,
