@@ -1,26 +1,31 @@
+import importlib
 import inspect
-import llvmlite.binding as ll
 import multiprocessing
-import numpy as np
 import os
 import shutil
+import stat
 import subprocess
 import sys
 import traceback
 import unittest
 import warnings
+import zipfile
+from pathlib import Path
+
+import llvmlite.binding as ll
+import numpy as np
+
 from numba import njit
 from numba.core import codegen
-from numba.core.caching import _UserWideCacheLocator
+from numba.core.caching import _UserWideCacheLocator, _ZipCacheLocator
 from numba.core.errors import NumbaWarning
 from numba.parfors import parfor
 from numba.tests.support import (
-    TestCase,
     SerialMixin,
+    TestCase,
     capture_cache_log,
     import_dynamic,
     override_config,
-    override_env_config,
     run_in_new_process_caching,
     skip_if_typeguard,
     skip_parfors_unsupported,
@@ -206,6 +211,7 @@ class BaseCacheTest(TestCase):
         self.modfile = os.path.join(self.tempdir, self.modname + ".py")
         self.cache_dir = os.path.join(self.tempdir, "__pycache__")
         shutil.copy(self.usecases_file, self.modfile)
+        os.chmod(self.modfile, stat.S_IREAD | stat.S_IWRITE)
         self.maxDiff = None
 
     def tearDown(self):
@@ -253,7 +259,7 @@ class DispatcherCacheUsecasesTest(BaseCacheTest):
     usecases_file = os.path.join(here, "cache_usecases.py")
     modname = "dispatcher_caching_test_fodder"
 
-    def run_in_separate_process(self):
+    def run_in_separate_process(self, *, envvars={}):
         # Cached functions can be run from a distinct process.
         # Also stresses issue #1603: uncached function calling cached function
         # shouldn't fail compiling.
@@ -265,8 +271,11 @@ class DispatcherCacheUsecasesTest(BaseCacheTest):
             mod.self_test()
             """ % dict(tempdir=self.tempdir, modname=self.modname)
 
+        subp_env = os.environ.copy()
+        subp_env.update(envvars)
         popen = subprocess.Popen([sys.executable, "-c", code],
-                                 stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+                                 stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+                                 env=subp_env)
         out, err = popen.communicate()
         if popen.returncode != 0:
             raise AssertionError(
@@ -313,10 +322,6 @@ class TestCache(DispatcherCacheUsecasesTest):
         self.check_pycache(9)  # 3 index, 6 data
         self.check_hits(f, 0, 2)
 
-        f = mod.generated_usecase
-        self.assertPreciseEqual(f(3, 2), 1)
-        self.assertPreciseEqual(f(3j, 2), 2 + 3j)
-
         # Check the code runs ok from another process
         self.run_in_separate_process()
 
@@ -457,7 +462,6 @@ class TestCache(DispatcherCacheUsecasesTest):
         mod.outer(2, 3)
         mod.record_return(mod.packed_arr, 0)
         mod.record_return(mod.aligned_arr, 1)
-        mod.generated_usecase(2, 3)
         mtimes = self.get_cache_mtimes()
         # Two signatures compiled
         self.check_hits(mod.add_usecase, 0, 2)
@@ -703,6 +707,92 @@ class TestCache(DispatcherCacheUsecasesTest):
         self.assertIn("cache hits = 1", err.strip())
 
 
+class TestCacheZip(DispatcherCacheUsecasesTest):
+
+    def setUp(self):
+        super().setUp()
+
+        # Create a simple Python module to be zipped
+        mod_content = """
+from numba import jit
+
+@jit(cache=True)
+def add(x, y):
+    return x + y
+"""
+        mod_filename = "test_module.py"
+        zip_filename = "test_archive.zip"
+
+        # Create a zip file containing the module
+        zip_path = os.path.join(self.tempdir, zip_filename)
+        with zipfile.ZipFile(zip_path, "w") as zf:
+            zf.writestr(mod_filename, mod_content)
+
+        # Add the zip file to sys.path
+        sys.path.insert(0, zip_path)
+        self.modname = "test_module"
+
+    def tearDown(self):
+        # Clean up: remove the zip file from sys.path
+        sys.path.pop(0)
+        # Remove the module from sys.modules to clean up
+        sys.modules.pop("test_module", None)
+
+    def test_zip_caching(self):
+        # (note that `self.import_module()` fails because its checks are
+        # incompatible
+        # with the zip file, so we just use normal imports here)
+
+        # First import and call
+        import test_module  # type: ignore
+
+        result1 = test_module.add(2, 3)
+        self.assertEqual(result1, 5)
+        self.check_hits(test_module.add, 0, 1)
+
+        # Record the initial cache hits
+        self.check_hits(test_module.add, 0)
+
+        # Remove the module and reimport
+        del sys.modules["test_module"]
+        importlib.invalidate_caches()
+        import test_module  # type: ignore
+
+        # Second call: should use the cache
+        result2 = test_module.add(2, 3)
+        self.assertEqual(result2, 5)
+
+        # Check if the cache was hit
+        self.check_hits(test_module.add, 1)
+
+
+class TestCacheZipLib(DispatcherCacheUsecasesTest):
+    """
+    ZipCache tests that don't require the setup/teardown from `TestCacheZip`
+    """
+    def test_zip_locator_creation(self):
+
+        def mock_func():
+            pass
+
+        zip_path = "/path/to/archive.zip/module.py"
+
+        locator = _ZipCacheLocator.from_function(mock_func, zip_path)
+        self.assertIsNotNone(locator)
+        self.assertEqual(locator._zip_path, str(Path("/path/to/archive.zip")))
+        self.assertEqual(locator._internal_path, "module.py")
+
+    def test_zip_locator_non_zip_path(self):
+
+        def mock_func():
+            pass
+
+        non_zip_path = "/path/to/module.py"
+
+        locator = _ZipCacheLocator.from_function(mock_func, non_zip_path)
+        self.assertIsNone(locator)
+
+
 @skip_parfors_unsupported
 class TestSequentialParForsCache(DispatcherCacheUsecasesTest):
     def setUp(self):
@@ -748,8 +838,7 @@ class TestCacheWithCpuSetting(DispatcherCacheUsecasesTest):
 
         mtimes = self.get_cache_mtimes()
         # Change CPU name to generic
-        with override_env_config('NUMBA_CPU_NAME', 'generic'):
-            self.run_in_separate_process()
+        self.run_in_separate_process(envvars={'NUMBA_CPU_NAME': 'generic'})
 
         self.check_later_mtimes(mtimes)
         self.assertGreater(len(self.cache_contents()), cache_size)
@@ -781,8 +870,9 @@ class TestCacheWithCpuSetting(DispatcherCacheUsecasesTest):
         system_features = codegen.get_host_cpu_features()
 
         self.assertNotEqual(system_features, my_cpu_features)
-        with override_env_config('NUMBA_CPU_FEATURES', my_cpu_features):
-            self.run_in_separate_process()
+        self.run_in_separate_process(
+            envvars={'NUMBA_CPU_FEATURES': my_cpu_features},
+        )
         self.check_later_mtimes(mtimes)
         self.assertGreater(len(self.cache_contents()), cache_size)
         # Check cache index
