@@ -5,7 +5,8 @@ import numpy
 import unittest
 from numba import njit, jit
 from numba.core.errors import TypingError, UnsupportedError
-from numba.tests.support import TestCase
+from numba.core import ir
+from numba.tests.support import TestCase, IRPreservingTestPipeline
 
 
 class TestClosure(TestCase):
@@ -364,7 +365,7 @@ class TestInlinedClosure(TestCase):
             return inner2(inner, x)
 
         def outer20(x):
-            #""" Test calling numpy in closure """
+            """ Test calling numpy in closure """
             z = x + 1
 
             def inner(x):
@@ -372,13 +373,20 @@ class TestInlinedClosure(TestCase):
             return inner(x)
 
         def outer21(x):
-            #""" Test calling numpy import as in closure """
+            """ Test calling numpy import as in closure """
             z = x + 1
 
             def inner(x):
                 return x + np.cos(z)
             return inner(x)
 
+        def outer22():
+            """Test to ensure that unsupported *args raises correctly"""
+            def bar(a, b):
+                pass
+            x = 1, 2
+            bar(*x)
+
         # functions to test that are expected to pass
         f = [outer1, outer2, outer5, outer6, outer7, outer8,
              outer9, outer10, outer12, outer13, outer14,
@@ -392,13 +400,13 @@ class TestInlinedClosure(TestCase):
         with self.assertRaises(NotImplementedError) as raises:
             cfunc = jit(nopython=True)(outer3)
             cfunc(var)
-        msg = "Unsupported use of op_LOAD_CLOSURE encountered"
+        msg = "Unsupported use of cell variable encountered"
         self.assertIn(msg, str(raises.exception))
 
         with self.assertRaises(NotImplementedError) as raises:
             cfunc = jit(nopython=True)(outer4)
             cfunc(var)
-        msg = "Unsupported use of op_LOAD_CLOSURE encountered"
+        msg = "Unsupported use of cell variable encountered"
         self.assertIn(msg, str(raises.exception))
 
         with self.assertRaises(TypingError) as raises:
@@ -425,102 +433,74 @@ class TestInlinedClosure(TestCase):
         msg = "The use of yield in a closure is unsupported."
         self.assertIn(msg, str(raises.exception))
 
+        with self.assertRaises(UnsupportedError) as raises:
+            cfunc = jit(nopython=True)(outer22)
+            cfunc()
+        msg = "Calling a closure with *args is unsupported."
+        self.assertIn(msg, str(raises.exception))
 
-class TestObjmodeFallback(TestCase):
-    # These are all based on tests from real life issues where, predominantly,
-    # the object mode fallback compilation path would fail as a result of the IR
-    # being mutated by closure inlining in npm. Tests are named after issues,
-    # all of which failed to compile as of 0.44.
+    def test_closure_renaming_scheme(self):
+        # See #7380, this checks that inlined (from closure) variables have a
+        # name derived from the function they were defined in.
 
-    decorators = [jit, jit(forceobj=True)]
+        @njit(pipeline_class=IRPreservingTestPipeline)
+        def foo(a, b):
+            def bar(z):
+                x = 5
+                y = 10
+                return x + y + z
+            return bar(a), bar(b)
 
-    def test_issue2955(self):
+        self.assertEqual(foo(10, 20), (25, 35))
 
-        def numbaFailure(scores, cooc):
-            rows, cols = scores.shape
-            for i in range(rows):
-                coxv = scores[i]
-                groups = sorted(set(coxv), reverse=True)
-                [set(np.argwhere(coxv == x).flatten()) for x in groups]
+        # check IR. Look for the `x = 5`... there should be
+        # Two lots of `const(int, 5)`, one for each inline
+        # The LHS of the assignment will have a name like:
+        # closure__locals__bar_v2_x
+        # Ensure that this is the case!
+        func_ir = foo.overloads[foo.signatures[0]].metadata['preserved_ir']
+        store = []
+        for blk in func_ir.blocks.values():
+            for stmt in blk.body:
+                if isinstance(stmt, ir.Assign):
+                    if isinstance(stmt.value, ir.Const):
+                        if stmt.value.value == 5:
+                            store.append(stmt)
 
-        x = np.random.random((10, 10))
-        y = np.abs((np.random.randn(10, 10) * 1.732)).astype(np.int)
-        for d in self.decorators:
-            d(numbaFailure)(x, y)
+        self.assertEqual(len(store), 2)
+        for i in store:
+            name = i.target.name
+            regex = r'closure__locals__bar_v[0-9]+.x'
+            self.assertRegex(name, regex)
 
-    def test_issue3239(self):
+    def test_issue9222(self):
+        # Ensures that float default arguments are handled correctly in
+        # closures.
 
-        def fit(X, y):
-            if type(X) is not np.ndarray:
-                X = np.array(X)
+        @njit
+        def foo():
+            def bar(x, y=1.1):
+                return x + y
+            return bar
 
-            if type(y) is not np.ndarray:
-                y = np.array(y)
+        @njit
+        def consume():
+            return foo()(4)
 
-            m, _ = X.shape
-            X = np.hstack((
-                np.array([[1] for _ in range(m)]),
-                X
-            ))
+        # In Issue #9222, the result was completely wrong - 15 instead of 5.1 -
+        # so allclose should be sufficient for comparison here.
+        np.testing.assert_allclose(consume(), 4 + 1.1)
 
-            res = np.dot(np.dot(X, X.T), y)
-            intercept = res[0]
-            coefs = res[1:]
-            return intercept, coefs
+    @TestCase.run_test_in_subprocess
+    def test_issue_9577(self):
+        @njit
+        def _inner():
+            range_start = 0
+            for _ in range(1):
+                np.array([1 for _ in range(range_start, 7)])
+                range_start = 0
 
-        for d in self.decorators:
-            res = d(fit)(np.arange(10).reshape(1, 10),
-                         np.arange(10).reshape(1, 10))
-            exp = fit(np.arange(10).reshape(1, 10),
-                      np.arange(10).reshape(1, 10))
-            np.testing.assert_equal(res, exp)
-
-    def test_issue3289(self):
-        b = [(5, 124), (52, 5)]
-
-        def a():
-            [b[index] for index in [0, 1]]
-            for x in range(5):
-                pass
-        for d in self.decorators:
-            d(a)()
-
-    def test_issue3413(self):
-
-        def foo(data):
-            # commenting out this line prevents the crash:
-            t = max([len(m) for m in data['y']])
-
-            mask = data['x'] == 0
-            if any(mask):
-                z = 15
-            return t, z
-
-        data = {'x': np.arange(5), 'y': [[1], [2, 3]]}
-        for d in self.decorators:
-            res = d(foo)(data)
-            np.testing.assert_allclose(res, foo(data))
-
-    def test_issue3659(self):
-
-        def main():
-            a = np.array(((1, 2), (3, 4)))
-            return np.array([x for x in a])
-        for d in self.decorators:
-            res = d(main)()
-            np.testing.assert_allclose(res, main())
-
-    def test_issue3803(self):
-
-        def center(X):
-            np.array([np.float_(x) for x in X.T])
-            np.array([np.float_(1) for _ in X.T])
-            return X
-
-        X = np.zeros((10,))
-        for d in self.decorators:
-            res = d(center)(X)
-            np.testing.assert_allclose(res, center(X))
+        _inner()
 
 
 if __name__ == '__main__':
