@@ -247,7 +247,7 @@ class TestReportedSSAIssues(SSABaseTest):
             for i in range(1):
                 arr = arr.reshape(3 * 2)
                 arr = arr.reshape(3, 2)
-            return(arr)
+            return (arr)
 
         np.testing.assert_allclose(foo(np.zeros((3, 2))),
                                    foo.py_func(np.zeros((3, 2))))
@@ -498,6 +498,60 @@ class TestReportedSSAIssues(SSABaseTest):
         # One phi?
         self.assertEqual(phi_counter, [1])
 
+    def test_issue9242_use_not_dom_def(self):
+        from numba.core.ir import FunctionIR
+        from numba.core.compiler_machinery import (
+            AnalysisPass,
+            register_pass,
+        )
+
+        def check(fir: FunctionIR):
+            [blk, *_] = fir.blocks.values()
+            var = blk.scope.get("d")
+            defn = fir.get_definition(var)
+            self.assertEqual(defn.op, "phi")
+            self.assertIn(ir.UNDEFINED, defn.incoming_values)
+
+        @register_pass(mutates_CFG=False, analysis_only=True)
+        class SSACheck(AnalysisPass):
+            """
+            Check SSA on variable `d`
+            """
+
+            _name = "SSA_Check"
+
+            def __init__(self):
+                AnalysisPass.__init__(self)
+
+            def run_pass(self, state):
+                check(state.func_ir)
+                return False
+
+        class SSACheckPipeline(CompilerBase):
+            """Inject SSACheck pass into the default pipeline following the SSA
+            pass
+            """
+
+            def define_pipelines(self):
+                pipeline = DefaultPassBuilder.define_nopython_pipeline(
+                    self.state, "ssa_check_custom_pipeline")
+
+                pipeline._finalized = False
+                pipeline.add_pass_after(SSACheck, ReconstructSSA)
+
+                pipeline.finalize()
+                return [pipeline]
+
+        @njit(pipeline_class=SSACheckPipeline)
+        def py_func(a):
+            c = a > 0
+            if c:
+                d = a + 5  # d is only defined here; undef in the else branch
+
+            return c and d > 0
+
+        py_func(10)
+
 
 class TestSROAIssues(MemoryLeakMixin, TestCase):
     # This tests issues related to the SROA optimization done in lowering, which
