@@ -1,9 +1,7 @@
-from __future__ import print_function, absolute_import
-
 import numpy as np
 
-from numba import types
-from numba import unittest_support as unittest
+from numba.core import types
+import unittest
 
 
 class TestTypeNames(unittest.TestCase):
